package com.softslate.commerce.administrator.core;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.administrator.Administrator;
import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.customer.core.BaseForm;
import com.softslate.commerce.customer.core.FormUtils;

/**
 * Extends the <code>BaseForm</code> class to provide base functionality for
 * form classes in the administrator. Provides a getter and setter for the
 * current user's <code>Administrator</code> object.
 * 
 * @author David Tobey
 * 
 */
public class BaseAdministratorForm extends BaseForm {

	private static final long serialVersionUID = 3257846575916069433L;
	/**
	 * The <code>Administrator</code> object representing the current
	 * administrative user.
	 */
	private Administrator administrator = null;

	public Administrator getAdministrator() {
		return (this.administrator);
	}

	public void setAdministrator(Administrator administrator) {
		this.administrator = administrator;
	}

	public void initializeProperties(ActionMapping mapping,
			HttpServletRequest request) {
		super.initializeProperties(mapping, request);
		// Load in the administrator bean from the session scope
		setAdministrator((Administrator) getUser().getAdministrator());
	}

	private FormUtils formUtils = null;

	public FormUtils getFormUtils(HttpServletRequest request) {
		if (formUtils != null) {
			return formUtils;
		}
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		formUtils = (FormUtils) bof.createUtilsObject("formUtilsImplementer");
		return formUtils;
	}

	public Character containsUnsafeCharacter(String string,
			HttpServletRequest request) {
		Settings settings = (Settings) request.getAttribute("settings");
		String unsafeCharacters = (String) settings
				.getValue("unsafeCharacters");
		return getFormUtils(request).containsCharacter(string,
				unsafeCharacters);
	}

	public Map mapOfIncomingParams() throws Exception {
		// Prepare the parameters - only those present in the request
		Map incomingProperties = PropertyUtils.describe(this);
		getFormUtils(getRequest()).removeNullsFromMap(incomingProperties);
		getFormUtils(getRequest()).removeAllNotInRequestFromMap(incomingProperties, getRequest());
		return incomingProperties;
	}

	public boolean addingNewRecord() {
		return getRequest().getParameter("addSubmit") != null || getRequest().getAttribute("addSubmit") != null;
	}

	public boolean addingAndFieldIsBlank(String field) {
		return addingNewRecord() && (field == null || field.equals(""));
	}

	public boolean editingAndFieldIsBeingEmptiedOut(String field) {
		return !addingNewRecord() && field != null && field.equals("");
	}

	public boolean addingAndFieldIsBlank(Integer field) {
		return addingNewRecord() && (field == null || field == 0);
	}

	public boolean editingAndFieldIsBeingEmptiedOut(Integer field) {
		return !addingNewRecord() && field != null && field == 0;
	}

}
