package com.softslate.commerce.administrator.extender;

import java.io.File;
import java.io.StringWriter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.io.FileUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;

public class ExtensionHandler {

	static Log log = LogFactory.getLog(ExtensionHandler.class);

	private Extension extension = null;
	private VelocityEngine ve = null;
	private VelocityContext vc = null;

	public ExtensionHandler(Extension extension) {
		this.extension = extension;
		initializeVelocity();
	}

	public void performExtension() throws Exception {
		createNonAdminContents();
		if (extension.isCreateAdminInterface()) {
			createAdminContents();
		}
		Entity e = extension.getEntity();
		File outDir = extension.getOutputDirectory();
		String bosDir = outDir + e.getPathToPackageUnderBusinessObjects();
		String daosDir = outDir + e.getPathToPackageUnderDAOs();
		String adminDir = outDir + e.getPathToPackageUnderAdministrator();
		String jspDir = outDir + e.getPathToCustomAdministratorJSPs();
		String eventsDir = outDir + e.getPathToPackageUnderEvents();
		if (extension.getUpdateExistingEntity()) {
			appendContentsIntoFile(extension.entityContents, bosDir + e.getInterfaceName() + ".java");
			appendContentsIntoFile(extension.entityBeanContents, bosDir + e.getInterfaceName() + "Bean.java");
			appendContentsIntoFile(extension.entityBeanHBMXMLContents, bosDir + e.getInterfaceName() + "Bean.hbm.xml");
			// TODO handle foreign key refs in BasicEntityProcessor
			writeContentsToDisk(extension.entityAlterSQLContents, bosDir + e.getInterfaceName() + "_alter_mysql.sql");
			if (extension.isCreateAdminInterface()) {
				appendContentsIntoFile(extension.entityAddEditFormContents, adminDir + e.getInterfaceName()
						+ "AddEditForm.java");
				// TODO handle fields string array property in EntityForm
				appendContentsIntoFile(extension.entityFormContents, adminDir + e.getInterfaceName() + "Form.java");
				appendContentsIntoFile(extension.entityFormGutsContents, jspDir + e.getLowerName() + "FormGuts.jsp");
				// TODO handle foreign key refs in EntityAction
				// TODO handle foreign key refs in EntityAddFormAction
				// TODO handle foreign key refs in EntityEditAction
				// TODO handle foreign key refs in EntityEditFormAction
				// TODO handle foreign key refs in entityFieldColumns.jsp
			}
		} else {
			writeContentsToDisk(extension.entityContents, bosDir + e.getInterfaceName() + ".java");
			writeContentsToDisk(extension.entityBeanContents, bosDir + e.getInterfaceName() + "Bean.java");
			writeContentsToDisk(extension.entityBeanHBMXMLContents, bosDir + e.getInterfaceName() + "Bean.hbm.xml");
			writeContentsToDisk(extension.entityProcessorContents, bosDir + e.getInterfaceName() + "Processor.java");
			writeContentsToDisk(extension.entityBasicProcessorContents, bosDir + "Basic" + e.getInterfaceName()
					+ "Processor.java");
			writeContentsToDisk(extension.entityDAOContents, daosDir + e.getInterfaceName() + "DAO.java");
			writeContentsToDisk(extension.entityDAOHibernateContents, daosDir + e.getInterfaceName()
					+ "DAOHibernate.java");
			writeContentsToDisk(extension.entityGatewayDAOContents, daosDir + e.getInterfaceName() + "GatewayDAO.java");
			writeContentsToDisk(extension.entityGatewayDAOHibernateContents, daosDir + e.getInterfaceName()
					+ "GatewayDAOHibernate.java");
			writeContentsToDisk(extension.entityAlterSQLContents, bosDir + e.getInterfaceName() + "_alter_mysql.sql");
			writeContentsToDisk(extension.entityCreateSQLContents, bosDir + e.getInterfaceName() + "_create_mysql.sql");
			writeContentsToDisk(extension.entityChangingEventContents, eventsDir + "Changing" + e.getInterfaceName()
					+ "Event.java");
			writeContentsToDisk(extension.entityChangingEventImplContents,
					eventsDir + "Changing" + e.getInterfaceName() + "EventImpl.java");
			writeContentsToDisk(extension.entityCRUDAPIContents, bosDir + e.getLowerName() + ".json.html");
			mergeContentsIntoFileIfItAlreadyExists(extension.entityAppComponentContents,
					outDir + System.getProperty("file.separator") + "src" + System.getProperty("file.separator")
							+ "appComponents-custom.properties", extension.getMergeTokenPropertiesStart(),
					extension.getMergeTokenPropertiesEnd());
			mergeContentsIntoFileIfItAlreadyExists(extension.entityQueriesContents,
					outDir + e.getPathToResourcesSubdirectory() + "queries-custom.hbm.xml",
					extension.getMergeTokenHtmlXmlStart(), extension.getMergeTokenHtmlXmlEnd());
			if (extension.isCreateAdminInterface()) {
				writeContentsToDisk(extension.entityActionContents, adminDir + e.getInterfaceName() + "Action.java");
				writeContentsToDisk(extension.entityAddActionContents, adminDir + e.getInterfaceName()
						+ "AddAction.java");
				writeContentsToDisk(extension.entityAddEditFormContents, adminDir + e.getInterfaceName()
						+ "AddEditForm.java");
				writeContentsToDisk(extension.entityAddFormActionContents, adminDir + e.getInterfaceName()
						+ "AddFormAction.java");
				writeContentsToDisk(extension.entityDeleteActionContents, adminDir + e.getInterfaceName()
						+ "DeleteAction.java");
				writeContentsToDisk(extension.entityEditActionContents, adminDir + e.getInterfaceName()
						+ "EditAction.java");
				writeContentsToDisk(extension.entityEditDeleteActionContents, adminDir + e.getInterfaceName()
						+ "EditDeleteAction.java");
				writeContentsToDisk(extension.entityEditFormActionContents, adminDir + e.getInterfaceName()
						+ "EditFormAction.java");
				writeContentsToDisk(extension.entityFormContents, adminDir + e.getInterfaceName() + "Form.java");
				writeContentsToDisk(extension.entityFormContents, adminDir + e.getInterfaceName() + "Form.java");
				writeContentsToDisk(extension.entityAddFormContents, jspDir + e.getLowerName() + "AddForm.jsp");
				writeContentsToDisk(extension.entityEditFormContents, jspDir + e.getLowerName() + "EditForm.jsp");
				writeContentsToDisk(extension.entityFormGutsContents, jspDir + e.getLowerName() + "FormGuts.jsp");
				writeContentsToDisk(extension.entityFieldColumnsContents, jspDir + e.getLowerName()
						+ "FieldColumns.jsp");
				writeContentsToDisk(extension.entityHeadingContents, jspDir + e.getLowerName() + "Heading.jsp");
				writeContentsToDisk(extension.entityExtraContents, jspDir + e.getLowerName() + "Extra.jsp");
				mergeContentsIntoFileIfItAlreadyExists(extension.entityNavigationLookupContents,
						outDir + e.getPathToCustomCoreAdministratorJSPs() + "navigationLookupDetails.jsp",
						extension.getMergeTokenJSPStart(), extension.getMergeTokenJSPEnd());
				mergeContentsIntoFileIfItAlreadyExists(extension.entitySideContents,
						outDir + e.getPathToCustomCoreAdministratorJSPs() + "side.jsp",
						extension.getMergeTokenJSPStart(), extension.getMergeTokenJSPEnd());
				mergeContentsIntoFileIfItAlreadyExists(extension.entityStrutsContents,
						outDir + e.getPathToCustomStrutsConfFile(), extension.getMergeTokenHtmlXmlStart(),
						extension.getMergeTokenHtmlXmlEnd());
				mergeContentsIntoFileIfItAlreadyExists(extension.entityTilesContents,
						outDir + e.getPathToCustomTilesConfFile(),
						extension.getMergeTokenHtmlXmlStart(), extension.getMergeTokenHtmlXmlEnd());
			}
		}
	}

	protected void initializeVelocity() {
		ve = new VelocityEngine();
		// ve.setProperty(VelocityEngine.RUNTIME_LOG_LOGSYSTEM, this);
		ve.setProperty("file.resource.loader.path", extension.getVelocityTemplateDirectory().getPath());
		ve.init();
		vc = new VelocityContext();
		vc.put("extension", extension);
		vc.put("entity", extension.getEntity());
		vc.put("escD", "$");
	}

	protected void createNonAdminContents() throws Exception {
		extension.entityContents = createFileContents("Entity.vm");
		extension.entityBeanContents = createFileContents("EntityBean.vm");
		extension.entityBeanHBMXMLContents = createFileContents("EntityBean.hbm.xml.vm");
		extension.entityAlterSQLContents = createFileContents("entity_customization_mysql.sql.vm");
		extension.entityCreateSQLContents = createFileContents("entity_create_mysql.sql.vm");
		extension.entityProcessorContents = createFileContents("EntityProcessor.vm");
		extension.entityBasicProcessorContents = createFileContents("EntityBasicProcessor.vm");
		extension.entityDAOContents = createFileContents("EntityDAO.vm");
		extension.entityDAOHibernateContents = createFileContents("EntityDAOHibernate.vm");
		extension.entityGatewayDAOContents = createFileContents("EntityGatewayDAO.vm");
		extension.entityGatewayDAOHibernateContents = createFileContents("EntityGatewayDAOHibernate.vm");
		extension.entityAppComponentContents = createFileContents("appComponents-custom.properties.vm");
		extension.entityQueriesContents = createFileContents("queries-custom.hbm.xml.vm");
		extension.entityChangingEventContents = createFileContents("EntityChangingEvent.vm");
		extension.entityChangingEventImplContents = createFileContents("EntityChangingEventImpl.vm");
		extension.entityCRUDAPIContents = createFileContents("entity.json.html.vm");
	}

	protected void createAdminContents() throws Exception {
		extension.entityActionContents = createFileContents("EntityAction.vm");
		extension.entityAddActionContents = createFileContents("EntityAddAction.vm");
		extension.entityAddEditFormContents = createFileContents("EntityAddEditForm.vm");
		extension.entityAddFormActionContents = createFileContents("EntityAddFormAction.vm");
		extension.entityDeleteActionContents = createFileContents("EntityDeleteAction.vm");
		extension.entityEditActionContents = createFileContents("EntityEditAction.vm");
		extension.entityEditDeleteActionContents = createFileContents("EntityEditDeleteAction.vm");
		extension.entityEditFormActionContents = createFileContents("EntityEditFormAction.vm");
		extension.entityFormContents = createFileContents("EntityForm.vm");
		extension.entityAddFormContents = createFileContents("entityAddForm.jsp.vm");
		extension.entityEditFormContents = createFileContents("entityEditForm.jsp.vm");
		extension.entityFormGutsContents = createFileContents("entityFormGuts.jsp.vm");
		extension.entityFieldColumnsContents = createFileContents("entityFieldColumns.jsp.vm");
		extension.entityHeadingContents = createFileContents("entityHeading.jsp.vm");
		extension.entityExtraContents = createFileContents("entityExtra.jsp.vm");
		extension.entityStrutsContents = createFileContents("struts-config-custom.xml.vm");
		extension.entityTilesContents = createFileContents("tiles-defs-custom.xml.vm");
		extension.entityNavigationLookupContents = createFileContents("navigationLookupDetails.jsp.vm");
		extension.entitySideContents = createFileContents("side.jsp.vm");
	}

	protected String createFileContents(String template) throws Exception {
		Template t = ve.getTemplate(template);
		StringWriter w = new StringWriter();
		t.merge(vc, w);
		log.info("Merged content from " + template);
		log.debug("Merged content from " + template + ": \n" + w.toString());
		return w.toString();
	}

	protected void writeContentsToDisk(String fileContents, String filePath) throws Exception {
		log.info("Writing content to " + filePath);
		File destinationFile = new File(filePath);
		destinationFile.getParentFile().mkdirs();
		FileUtils.writeStringToFile(destinationFile, fileContents, "UTF-8");
	}

	protected void mergeContentsIntoFileIfItAlreadyExists(String fileContents, String filePath, String mergeTokenStart,
			String mergeTokenEnd) throws Exception {
		File destinationFile = new File(filePath);
		if (destinationFile.exists()) {
			log.info("Merging content into " + destinationFile.getPath());
			String existingContents = FileUtils.readFileToString(destinationFile, "UTF-8");
			fileContents = mergeExistingFileContents(fileContents, existingContents, mergeTokenStart, mergeTokenEnd);
		}
		writeContentsToDisk(fileContents, filePath);
	}

	protected String mergeExistingFileContents(String fileContents, String existingContents,
			String destinationTokenStart, String destinationTokenEnd) throws Exception {
		String regEx = "(" + destinationTokenStart + ".*?" + destinationTokenEnd + ")";
		Pattern pattern = Pattern.compile(regEx, Pattern.DOTALL);
		Matcher matcherExisting = pattern.matcher(existingContents);
		if (matcherExisting.find()) {
			existingContents = matcherExisting.replaceAll("");
		}
		Matcher matcher = pattern.matcher(fileContents);
		while (matcher.find()) {
			String replacement = matcher.group(1);
			if (replacement.contains("<form-bean")) {
				existingContents = existingContents.replaceFirst("</form-beans>",
						Matcher.quoteReplacement("\t" + replacement + "\n\t</form-beans>"));
			} else if (replacement.contains("<action")) {
				existingContents = existingContents.replaceFirst("</action-mappings>",
						Matcher.quoteReplacement("\t" + replacement + "\n\t</action-mappings>"));
			} else if (replacement.contains("<definition")) {
				existingContents = existingContents.replaceFirst("</tiles-definitions>",
						Matcher.quoteReplacement("\t" + replacement + "\n</tiles-definitions>"));
			} else if (replacement.contains("<query")) {
				existingContents = existingContents.replaceFirst("</hibernate-mapping>",
						Matcher.quoteReplacement("\t" + replacement + "\n</hibernate-mapping>"));
			} else if (replacement.contains("<c:when")) {
				existingContents = replaceLast(existingContents, "</c:choose>", "\t" + replacement + "\n\t</c:choose>");
			} else if (replacement.contains("<logic:present role=")) {
				existingContents = existingContents
						.replaceFirst(
								"<ul id=\"ExtensionsMenu\" class=\"menu\">",
								Matcher.quoteReplacement("<ul id=\"ExtensionsMenu\" class=\"menu\">\n\t\t\t\t\t"
										+ replacement));
			} else {
				existingContents = existingContents + "\n" + replacement;
			}
		}
		return existingContents;
	}

	protected void appendContentsIntoFile(String fileContents, String filePath) throws Exception {
		File destinationFile = new File(filePath);
		if (destinationFile.exists()) {
			log.info("Appending content into " + destinationFile.getPath());
			String existingContents = FileUtils.readFileToString(destinationFile, "UTF-8");
			if (filePath.endsWith(".java")) {
				fileContents = appendContentsToExistingFileContents(fileContents, existingContents,
						extension.getMergeTokenJavaStart(), extension.getMergeTokenJavaEnd(), "}");
			} else if (filePath.endsWith("Bean.hbm.xml")) {
				fileContents = appendContentsToExistingFileContents(fileContents, existingContents,
						extension.getMergeTokenHtmlXmlStart(), extension.getMergeTokenHtmlXmlEnd(), "<subclass");
			} else if (filePath.endsWith(".jsp")) {
				fileContents = appendContentsToExistingFileContents(fileContents, existingContents,
						extension.getMergeTokenJSPStart(), extension.getMergeTokenJSPEnd(), "<!--");
			}
		}
		writeContentsToDisk(fileContents, filePath);
	}

	protected String appendContentsToExistingFileContents(String fileContents, String existingContents,
			String newFileTokenStart, String newFileTokenEnd, String endingMarker) throws Exception {
		String regEx = newFileTokenStart + "(.*?)" + newFileTokenEnd;
		Pattern pattern = Pattern.compile(regEx, Pattern.DOTALL);
		Matcher matcher = pattern.matcher(fileContents);
		while (matcher.find()) {
			String toAppend = matcher.group(1);
			existingContents = replaceLast(existingContents, endingMarker, toAppend + "\n" + endingMarker);
		}
		return existingContents;
	}

	public static String replaceLast(String original, String toReplace, String replacement) {
		StringBuilder b = new StringBuilder(original);
		b.replace(original.lastIndexOf(toReplace), original.lastIndexOf(toReplace) + toReplace.length(), replacement);
		return b.toString();
	}
}
