package com.softslate.commerce.administrator.importexport;

import java.io.File;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.upload.FormFile;
import org.apache.struts.upload.MultipartRequestHandler;

import com.softslate.commerce.administrator.core.BaseSerializableForm;

/**
 * Action form class corresponding to the form where an import is described.
 * Corresponds to requests to /ImportUploadForm.do, /ImportPreviewProcess.do,
 * and /ImportProcess.do.
 * 
 * @author David Tobey, Ashish Jain
 * 
 */
public class ImportForm extends BaseSerializableForm {

	private static final long serialVersionUID = 7928620557429852491L;

	static Log log = LogFactory.getLog(ImportForm.class);

	private String submitProcess = null;

	private String previewCount = "20";

	private String table = "";

	private String delimiter = "tab";

	private String excel = "true";

	private String useFileOnServer = "false";

	private String pathOnServer = null;

	private String importBehavior = "insertAndUpdate";

	private FormFile file = null;

	private String headerRow = "true";

	private String overwrite = "true";

	private String[] fields = null;

	private Collection fieldOptions = null;

	private Collection records = null;

	private String[] parsedHeaderFields = null;

	public String getSubmitProcess() {
		return submitProcess;
	}

	public void setSubmitProcess(String submitProcess) {
		this.submitProcess = submitProcess;
	}

	public String getPreviewCount() {
		return previewCount;
	}

	public void setPreviewCount(String previewCount) {
		this.previewCount = previewCount;
	}

	public String getDelimiter() {
		return delimiter;
	}

	public void setDelimiter(String delimiter) {
		this.delimiter = delimiter;
	}

	public String getExcel() {
		return excel;
	}

	public void setExcel(String excel) {
		this.excel = excel;
	}

	public String getTable() {
		return table;
	}

	public void setTable(String table) {
		this.table = table;
	}

	public String getUseFileOnServer() {
		return useFileOnServer;
	}

	public void setUseFileOnServer(String useFileOnServer) {
		this.useFileOnServer = useFileOnServer;
	}

	public String getPathOnServer() {
		return pathOnServer;
	}

	public void setPathOnServer(String pathOnServer) {
		this.pathOnServer = pathOnServer;
	}

	public FormFile getFile() {
		return file;
	}

	public void setFile(FormFile file) {
		this.file = file;
	}

	public Collection getFieldOptions() {
		return fieldOptions;
	}

	public void setFieldOptions(Collection fieldOptions) {
		this.fieldOptions = fieldOptions;
	}

	public String[] getFields() {
		return fields;
	}

	public void setFields(String[] fields) {
		this.fields = fields;
	}

	public String getHeaderRow() {
		return headerRow;
	}

	public void setHeaderRow(String headerRow) {
		this.headerRow = headerRow;
	}

	public Collection getRecords() {
		return records;
	}

	public void setRecords(Collection records) {
		this.records = records;
	}

	public String getImportBehavior() {
		return importBehavior;
	}

	public void setImportBehavior(String importBehavior) {
		this.importBehavior = importBehavior;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isDebugEnabled())
			log.debug("Starting validation.");

		ActionErrors errors = new ActionErrors();

		Boolean maxLengthExceeded = (Boolean) request
				.getAttribute(MultipartRequestHandler.ATTRIBUTE_MAX_LENGTH_EXCEEDED);

		if ((maxLengthExceeded != null) && (maxLengthExceeded.booleanValue())) {
			if (log.isInfoEnabled())
				log.info("File too large");
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"errors.importFileTooLarge"));
		}

		if (getDelimiter() == null || getDelimiter().equals("")) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"errors.required", "Delimiter"));
		}

		if (getTable() == null || getTable().equals("")) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"errors.required", "Data to Import"));
		}

		if (getSubmitProcess() == null) {
			if (!((getUseFileOnServer().equals("false") && getFile() != null && getFile()
					.getFileSize() > 0) || (getPathOnServer() != null && getPathOnServer()
					.length() > 0))) {
				errors
						.add(
								ActionMessages.GLOBAL_MESSAGE,
								new ActionMessage("errors.detail",
										"You must either select a file to upload or use an existing file."));
			}
		} else {
			if (getFields() == null
					|| (getFields().length == 1 && getFields()[0].equals(""))) {
				errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"errors.detail", "No fields submitted."));

			}
			if (getFields() != null) {
				boolean empty = true;
				for (int i = 0; i < getFields().length; i++) {
					if (getFields()[i] != null && getFields()[i].length() > 0) {
						empty = false;
					}
				}
				if (empty) {
					errors.add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.detail",
									"No fields submitted."));
				}
			}
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		return errors;

	}

	public void reset(ActionMapping mapping, HttpServletRequest request) {
		setSubmitProcess(null);
		if (getPathOnServer() == null) {
			File tempDir = (File) getServlet().getServletContext()
					.getAttribute("javax.servlet.context.tempdir");
			String pathOnServer = (new File(tempDir, "importFile.txt"))
					.toString();
			setPathOnServer(pathOnServer);
		}
	}

	public String getOverwrite() {
		return overwrite;
	}

	public void setOverwrite(String overwrite) {
		this.overwrite = overwrite;
	}

	public String[] getParsedHeaderFields() {
		return parsedHeaderFields;
	}

	public void setParsedHeaderFields(String[] parsedFields) {
		this.parsedHeaderFields = parsedFields;
	}
}
