package com.softslate.commerce.administrator.order;

import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.ApplicationMessage;
import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/OrderProcessSelected.do". Processes a request to update the
 * status for, or prepare to print, a set of selected orders.
 * 
 * @author David Tobey
 * 
 */
public class OrderProcessSelectedAction extends BaseAction {

	static Log log = LogFactory.getLog(OrderProcessSelectedAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		OrderForm baseForm = (OrderForm) form;
		ActionMessages messages = new ActionMessages();
		ActionMessages errors = new ActionMessages();

		boolean valid = verifySomeRecordsWereSelected(baseForm, errors) && verifyAnActionWasSelected(baseForm, errors)
				&& verifyAStatusWasSelected(baseForm, errors) && verifyAPrintTemplateWasSelected(baseForm, errors)
				&& verifyAnEmailTemplateWasSelected(baseForm, errors);
		if (!valid) {
			saveErrors(request, (ActionMessages) errors);
			if (log.isInfoEnabled() && !errors.isEmpty()) {
				log.info("Validation error(s): " + errors.toString());
			}
			return mapping.findForward("failure");
		}

		// Process updating the statuses of the selected records
		if (baseForm.getWithSelectedAction().equals("Change") && baseForm.getWithSelectedStatus() != null
				&& baseForm.getWithSelectedStatus().length() > 0) {
			String result = processStatusChanges(baseForm, request, messages, errors);
			return mapping.findForward(result);
		}

		// Process printing multiple records
		if (baseForm.getWithSelectedAction().equals("Print") && baseForm.getWithSelectedPrint() != null
				&& baseForm.getWithSelectedPrint().length() > 0) {
			return mapping.findForward("successPrint");
		}

		// Process sending the emails of the selected records
		if (baseForm.getWithSelectedAction().equals("Send Emails") && baseForm.getWithSelectedEmail() != null
				&& baseForm.getWithSelectedEmail().length() > 0) {
			String result = processEmails(baseForm, request, mapping, messages, errors);
			return mapping.findForward(result);
		}

		return mapping.findForward("success");
	}

	protected boolean verifySomeRecordsWereSelected(OrderForm baseForm, ActionMessages errors) {
		if (baseForm.getSelectedID() == null || baseForm.getSelectedID().length == 0) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail",
					"Use the checkboxes to select at least one order."));
			return false;
		}
		return true;
	}

	protected boolean verifyAnActionWasSelected(OrderForm baseForm, ActionMessages errors) {
		if (baseForm.getWithSelectedAction() == null || baseForm.getWithSelectedAction().length() == 0) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail", "No action identified."));
			return false;
		}
		return true;
	}

	protected boolean verifyAStatusWasSelected(OrderForm baseForm, ActionMessages errors) {
		if (baseForm.getWithSelectedAction().equals("Change")
				&& (baseForm.getWithSelectedStatus() == null || baseForm.getWithSelectedStatus().length() == 0)) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail",
					"Identify the status to change to in the drop down menu."));
			return false;
		}
		return true;
	}

	protected boolean verifyAPrintTemplateWasSelected(OrderForm baseForm, ActionMessages errors) {
		if (baseForm.getWithSelectedAction().equals("Print")
				&& (baseForm.getWithSelectedPrint() == null || baseForm.getWithSelectedPrint().length() == 0)) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail",
					"Identify the print template in the drop down menu."));
			return false;
		}
		return true;
	}

	protected boolean verifyAnEmailTemplateWasSelected(OrderForm baseForm, ActionMessages errors) {
		if (baseForm.getWithSelectedAction().equals("Send Emails")
				&& (baseForm.getWithSelectedEmail() == null || baseForm.getWithSelectedEmail().length() == 0)) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail",
					"Identify the email template in the drop down menu."));
			return false;
		}
		return true;
	}

	protected String processStatusChanges(OrderForm baseForm, HttpServletRequest request, ActionMessages messages,
			ActionMessages errors) throws Exception {
		BusinessObjectFactory bof = (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
		OrderProcessor processor =  bof.createObject(OrderProcessor.class);

		// Enforce role permissions - struts-config-order.xml allows
		// orderView, so we should enforce orderEdit here.
		if (!request.isUserInRole("superuser") && !request.isUserInRole("administrator")
				&& !request.isUserInRole("orderEdit")) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.administratorForbidden"));
			saveErrors(request, (ActionMessages) errors);
			if (log.isInfoEnabled() && !errors.isEmpty())
				log.info("Validation error(s): " + errors.toString());
			return "failure";
		}

		Map parameters = PropertyUtils.describe(baseForm);
		String[] displayFields = { "status" };
		String[] orderID = baseForm.getSelectedID();
		String[] idValues = baseForm.getSelectedID();
		Integer[] idIntegers = (new Integer[idValues.length]);
		String[] status = new String[idValues.length];
		for (int i = 0; i < idValues.length; i++) {
			if (idValues[i].length() > 0) {
				idIntegers[i] = new Integer(Integer.parseInt(idValues[i]));
				status[i] = baseForm.getWithSelectedStatus();
			}
		}
		parameters.put("displayFields", displayFields);
		parameters.put("orderID", orderID);
		parameters.put("idValues", idValues);
		parameters.put("idIntegers", idIntegers);
		parameters.put("status", status);
		parameters.put("deleteID", new String[] {});
		Map results = processor.editDeleteOrders(parameters);
		// Set the success message
		messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordsSaved"));
		ActionUtils actionUtils = (ActionUtils) bof.createUtilsObject("actionUtilsImplementer");
		actionUtils.processApplicationMessages(errors, messages,
				(Collection<ApplicationMessage>) results.get("applicationMessages"));
		saveMessages(request, messages);
		return "success";
	}

	protected String processEmails(OrderForm baseForm, HttpServletRequest request, ActionMapping mapping,
			ActionMessages messages, ActionMessages errors) throws Exception {

		// Enforce role permissions - struts-config-order.xml allows
		// orderView, so we should enforce orderEdit here.
		if (!request.isUserInRole("superuser") && !request.isUserInRole("administrator")
				&& !request.isUserInRole("orderEdit")) {
			errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.administratorForbidden"));
			saveErrors(request, (ActionMessages) errors);
			if (log.isInfoEnabled() && !errors.isEmpty())
				log.info("Validation error(s): " + errors.toString());
			return "failure";
		}

		// Single orderMultipleEmail.jsp to send emails
		request.setAttribute("sendMultipleEmails", new Boolean(true));
		BaseForm newBaseForm = new BaseForm();
		newBaseForm.initializeProperties(mapping, request);
		request.setAttribute("baseForm", newBaseForm);

		// Set the success message
		messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.detail", "The emails are being sent."));
		saveMessages(request, messages);
		return "successEmail";
	}
}
