package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * <code>/administrator/CategoryAddForm.do</code>. Creates an empty
 * <code>CategoryAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class CategoryAddFormAction extends BaseAction {

	static Log log = LogFactory.getLog(CategoryAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		CategoryAddEditForm baseForm = (CategoryAddEditForm) form;

		if (request.getAttribute("reset") != null
				&& request.getAttribute("reset").equals("true")) {
			Class classDefinition = baseForm.getClass();
			baseForm = (CategoryAddEditForm) classDefinition.newInstance();
			if (request.getParameter("workWithCategories") != null) {
				baseForm.setWorkWithCategories(request
						.getParameter("workWithCategories"));
			}
		}

		baseForm.initializeProperties(mapping, request);
		request.setAttribute("categoryAddEditForm", baseForm);

		// Set content page defaults
		if (baseForm.getWorkWithCategories() != null
				&& baseForm.getWorkWithCategories().equals("Content Pages")) {
			baseForm.setIsContentPage("true");
			baseForm.setCategoryTreeID("0");
		}

		// Load the parent if an id is present
		Product product = null;
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor productProcessor = (ProductProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"productProcessorImplementer");
			Map parameters = new HashMap();
			parameters.put("productID", baseForm.getProductID());
			product = productProcessor.loadProductFromID(parameters);
			request.setAttribute("product", product);
		}

		// Load all categories for the parent category drop down.
		CategoryProcessor processor = (CategoryProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"categoryProcessorImplementer");
		Collection categories = processor.loadAllCategories();
		baseForm.getRequest().setAttribute("categories", categories);

		// Navigation
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Categories", "Add Category" });
			if (baseForm.getWorkWithCategories() != null
					&& baseForm.getWorkWithCategories().equals("Content Pages")) {
				bcs = Arrays
						.asList(new String[] { "Home", "Products",
								"Product Details", "Content Pages",
								"Add Content Page" });
			}
			request.setAttribute("breadcrumbs", bcs);
		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Categories",
					"Add Category" });
			if (baseForm.getWorkWithCategories() != null
					&& baseForm.getWorkWithCategories().equals("Content Pages")) {
				bcs = Arrays.asList(new String[] { "Home", "Content Pages",
						"Add Content Page" });
			}
			request.setAttribute("breadcrumbs", bcs);
		}

		return mapping.findForward("success");
	}
}
