package com.softslate.commerce.administrator.shipping;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.shipping.ShippingMethod;
import com.softslate.commerce.businessobjects.shipping.ShippingMethodProcessor;
import com.softslate.commerce.businessobjects.shipping.ShippingRateProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the main administrator view panel for
 * shipping rates. Used for requests to "/administrator/ShippingRate.do".
 * <p>
 * The <code>execute</code> method of this class populates a
 * <code>Collection</code> named <code>itemCollection</code> in the request
 * scope containing the items to be displayed. Next, it sets a request scope
 * variable named <code>editPermission</code> that determines if the current
 * user has permission to edit the items. Finally, it creates a
 * <code>Collection</code> named <code>breadcrumbs</code> containing the
 * links and labels of each breadcrumb for display.
 * 
 * @author David Tobey
 * 
 */
public class ShippingRateAction extends BaseAction {

	static Log log = LogFactory.getLog(ShippingRateAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		
		ShippingRateForm baseForm = (ShippingRateForm) form;
		baseForm.setProperties(request);
		request.setAttribute("mainForm", baseForm);

		// Handle the quick-add form
		ShippingRateAddEditForm addForm = (ShippingRateAddEditForm) request
				.getAttribute("shippingRateAddEditForm");
		if (addForm == null
				|| (request.getAttribute("reset") != null && request
						.getAttribute("reset").equals("true"))) {
			addForm = new ShippingRateAddEditForm();
		}
		addForm.initializeProperties(mapping, request);
		request.setAttribute("shippingRateAddEditForm", addForm);

		// Load the parent if an id is present
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		ShippingMethod shippingMethod = null;
		if (baseForm.getShippingMethodID() != null
				&& baseForm.getShippingMethodID().length() > 0) {
			ShippingMethodProcessor processor =  bof
					.createObject(ShippingMethodProcessor.class);
			shippingMethod = processor.loadShippingMethodFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("shippingMethod", shippingMethod);
		}

		// Get the collection of items to display
		ShippingRateProcessor processor =  bof
				.createObject(ShippingRateProcessor.class);
		Map shippingRatesAndCount = processor
				.loadShippingRatesAndCount(PropertyUtils.describe(baseForm));
		request.setAttribute("shippingRates", shippingRatesAndCount
				.get("items"));
		request.setAttribute("itemCollection", shippingRatesAndCount
				.get("items"));
		request.setAttribute("itemCount", shippingRatesAndCount
				.get("itemCount"));

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("shippingEdit")) {
			request.setAttribute("editPermission", "false");
			baseForm.setEditMode("false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays
				.asList(new String[] { "Home", "Shipping Methods",
						"Shipping Method Details", "Shipping Rates" });
		request.setAttribute("breadcrumbs", bcs);

		// Forward according to the outputMode of the request
		if (baseForm.getOutputMode().equals("print")) {
			request.setAttribute("outputMode", "print");
			baseForm.setOutputMode("screen");
			return mapping.findForward("successSimpleLayout");
		} else if (baseForm.getOutputMode().equals("excel")) {
			request.setAttribute("outputMode", "excel");
			baseForm.setOutputMode("screen");
			response.setHeader("Content-Type", "application/vnd.ms-excel");
			response.setHeader("Content-Disposition",
					"attachment; filename=shippingRates.xls");
			return mapping.findForward("successSimpleLayout");
		}
		return mapping.findForward("success");
	}
}
