package com.softslate.commerce.businessobjects.administrator;

import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.administrator.RoleDAO;
import com.softslate.commerce.daos.administrator.RoleGatewayDAO;

/**
 * Processes business logic having to do with roles. Created in the Struts layer
 * in response to requests for viewing, editing, adding, and deleting roles.
 * <p>
 * <code>BasicRoleProcessor</code> is the default implementation of the
 * <code>RoleProcessor</code> interface for the application.
 * 
 * @author David Tobey
 *  
 */
public class BasicRoleProcessor extends BaseBusinessProcessor implements
        RoleProcessor {

    static Log log = LogFactory.getLog(BasicRoleProcessor.class);

    public Map loadRolesAndCount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading roles and count");
        RoleGatewayDAO gatewayDAO = (RoleGatewayDAO) getDaoFactory().createDAO(
                "roleGatewayDAOImplementer");
        return gatewayDAO.loadRolesAndCount(parameters);
    }

    public void addRole(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Adding role: " + code);
        }
        Role role = (Role) getBusinessObjectFactory().createObject(
                "roleImplementer");
        BeanUtils.copyProperties(role, parameters);
        RoleDAO dao =  getDaoFactory().createDAO(RoleDAO.class);
        dao.setRole(role);
        dao.insertRole(true);
        return;
    }

    public Role loadRoleFromID(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading role from id");
        Role role = (Role) getBusinessObjectFactory().createObject(
                "roleImplementer");
        RoleDAO dao =  getDaoFactory().createDAO(RoleDAO.class);
        BeanUtils.copyProperties(role, parameters);
        dao.setRole(role);
        dao.loadRoleFromID();
		return dao.getRole();
    }

    public void editRole(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing role: " + code);
        }
		Role role = loadRoleFromID(parameters);
        BeanUtils.copyProperties(role, parameters);
        RoleDAO dao =  getDaoFactory().createDAO(RoleDAO.class);
        dao.setRole(role);
        dao.updateRole(true);
        return;
    }

	public Map editDeleteRoles(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple roles");
		RoleGatewayDAO dao =  getDaoFactory().createDAO(RoleGatewayDAO.class);
		Map results = dao.editDeleteRoles(parameters, getBusinessObjectFactory());
		return results;
	}

    public void deleteRole(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
            String code = (String) parameters.get("code");
            log.info(getUser().getAdministrator().getUserName()
                    + ": Deleting role: " + code);
        }
        Role role = (Role) getBusinessObjectFactory().createObject(
                "roleImplementer");
        BeanUtils.copyProperties(role, parameters);
        RoleDAO dao =  getDaoFactory().createDAO(RoleDAO.class);
        dao.setRole(role);
        dao.deleteRole(true);
        return;
    }
}
