package com.softslate.commerce.businessobjects.administrator;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with roles. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting roles.
 * <p>
 * When the application needs to create an instance that implements
 * <code>RoleProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "roleProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "roleProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.administrator.RoleProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface RoleProcessor extends BusinessProcessor {

	abstract Map loadRolesAndCount(Map parameters) throws Exception;

	abstract void addRole(Map parameters) throws Exception;

	abstract Role loadRoleFromID(Map parameters) throws Exception;

	abstract void editRole(Map parameters) throws Exception;

	abstract Map editDeleteRoles(Map parameters) throws Exception;

	abstract void deleteRole(Map parameters) throws Exception;
}
