package com.softslate.commerce.businessobjects.core;

import java.io.Serializable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Represents a single entry in the request log, ie, a single request made by a
 * user to the application. <code>RequestLogBean</code> is the default
 * implementation of the <code>RequestLog</code> interface for the application.
 * <p>
 * The database stores the request log in the <code>sscRequestLog</code> table,
 * and this class can be used to represent a single row of that table. The
 * Hibernate configuration file <code>RequestLogBean.hbm.xml</code> maps that
 * table's columns to the properties in this object.
 * 
 * @author David Tobey
 */
public class RequestLogBean extends BaseBusinessProcessor implements
		Serializable, RequestLog {

	private static final long serialVersionUID = 1L;

	static Log log = LogFactory.getLog(RequestLogBean.class);

	private int requestLogID;
	private Integer orderID;
	private Integer orderNumber;
	private Integer customerID;
	private String customerUserName;
	private Integer administratorID;
	private String administratorUserName;
	private String sessionID;
	private String ipAddress;
	private String referrer;
	private String userAgent;
	private String servletPath;
	private String requestURL;
	private String requestParameters;
	private String searchString;
	private boolean isAdminRequest;
	private String created;

	public int getRequestLogID() {
		return requestLogID;
	}

	public void setRequestLogID(int requestLogID) {
		this.requestLogID = requestLogID;
	}

	public Integer getOrderID() {
		return orderID;
	}

	public void setOrderID(Integer orderID) {
		this.orderID = orderID;
	}

	public Integer getOrderNumber() {
		return orderNumber;
	}

	public void setOrderNumber(Integer orderNumber) {
		this.orderNumber = orderNumber;
	}

	public Integer getCustomerID() {
		return customerID;
	}

	public void setCustomerID(Integer customerID) {
		this.customerID = customerID;
	}

	public String getCustomerUserName() {
		return customerUserName;
	}

	public void setCustomerUserName(String customerUserName) {
		this.customerUserName = customerUserName;
	}

	public Integer getAdministratorID() {
		return administratorID;
	}

	public void setAdministratorID(Integer administratorID) {
		this.administratorID = administratorID;
	}

	public String getAdministratorUserName() {
		return administratorUserName;
	}

	public void setAdministratorUserName(String administratorUserName) {
		this.administratorUserName = administratorUserName;
	}

	public String getSessionID() {
		return sessionID;
	}

	public void setSessionID(String sessionID) {
		this.sessionID = sessionID;
	}

	public String getIpAddress() {
		return ipAddress;
	}

	public void setIpAddress(String ipAddress) {
		this.ipAddress = ipAddress;
	}

	public String getReferrer() {
		return referrer;
	}

	public void setReferrer(String referrer) {
		this.referrer = referrer;
	}

	public String getUserAgent() {
		return userAgent;
	}

	public void setUserAgent(String userAgent) {
		this.userAgent = userAgent;
	}

	public String getServletPath() {
		return servletPath;
	}

	public void setServletPath(String servletPath) {
		this.servletPath = servletPath;
	}

	public String getRequestURL() {
		return requestURL;
	}

	public void setRequestURL(String requestURL) {
		this.requestURL = requestURL;
	}

	public String getRequestParameters() {
		return requestParameters;
	}

	public void setRequestParameters(String requestParameters) {
		this.requestParameters = requestParameters;
	}

	public String getSearchString() {
		return searchString;
	}

	public void setSearchString(String searchString) {
		this.searchString = searchString;
	}

	public boolean getIsAdminRequest() {
		return isAdminRequest;
	}

	public void setIsAdminRequest(boolean isAdminRequest) {
		this.isAdminRequest = isAdminRequest;
	}

	public String getCreated() {
		return created;
	}

	public void setCreated(String created) {
		this.created = created;
	}

	public String toString() {
		StringBuffer auditMessage = new StringBuffer("");
		auditMessage.append("\t");
		auditMessage.append(getServletPath());
		auditMessage.append("\t");
		if (getSessionID() != null) {
			auditMessage.append(getSessionID());
		}
		auditMessage.append("\t");
		if (getIpAddress() != null) {
			auditMessage.append(getIpAddress());
		}
		auditMessage.append("\t");
		if (getOrderID() != null) {
			auditMessage.append(getOrderID());
		}
		auditMessage.append("\t");
		if (getCustomerUserName() != null) {
			auditMessage.append(getCustomerUserName());
		}
		auditMessage.append("\t");
		if (getAdministratorUserName() != null) {
			auditMessage.append(getAdministratorUserName());
		}
		auditMessage.append("\t");
		if (getReferrer() != null) {
			auditMessage.append(getReferrer());
		}
		auditMessage.append("\t");
		if (getRequestURL() != null) {
			auditMessage.append(getRequestURL());
		}
		auditMessage.append("\t");
		if (getRequestParameters() != null) {
			auditMessage.append(getRequestParameters());
		}
		return auditMessage.toString();
	}
}
