package com.softslate.commerce.businessobjects.order;

import java.util.Map;
import java.util.HashMap;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.order.OrderDAO;
import com.softslate.commerce.daos.order.OrderSettingDAO;

/**
 * Processes business logic having to do with order settings. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting order settings.
 * <p>
 * <code>BasicOrderSettingProcessor</code> is the default implementation of the
 * <code>OrderSettingProcessor</code> interface for the application.
 * 
 * @author Jason
 */
public class BasicOrderSettingProcessor extends BaseBusinessProcessor implements
		OrderSettingProcessor {

	static Log log = LogFactory.getLog(BasicOrderSettingProcessor.class);

	public void addOrderSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Adding order setting: " + code);
		}
		OrderSetting orderSetting = (OrderSetting) getBusinessObjectFactory()
				.createObject("orderSettingImplementer");
		BeanUtils.copyProperties(orderSetting, parameters);
		Order order = (Order) getBusinessObjectFactory().createObject(
				"orderImplementer");
		order.setOrderID(orderSetting.getOrderID());
		orderSetting.setOrder(order);
		OrderSettingDAO dao = (OrderSettingDAO) getDaoFactory().createDAO(
				"orderSettingDAOImplementer");
		dao.setOrderSetting(orderSetting);
		dao.insertOrderSetting(true);
		return;
	}

	public OrderSetting loadOrderSettingFromID(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading order setting from id");
		OrderSetting orderSetting = (OrderSetting) getBusinessObjectFactory()
				.createObject("orderSettingImplementer");
		OrderSettingDAO dao = (OrderSettingDAO) getDaoFactory().createDAO(
				"orderSettingDAOImplementer");
		BeanUtils.copyProperties(orderSetting, parameters);
		dao.setOrderSetting(orderSetting);
		dao.loadOrderSettingFromID();
		return orderSetting;
	}

	public OrderSetting loadOrderSettingObjectFromID(Map parameters)
			throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading order setting from id");
		OrderSetting orderSetting = (OrderSetting) getBusinessObjectFactory()
				.createObject("orderSettingImplementer");
		OrderSettingDAO dao = (OrderSettingDAO) getDaoFactory().createDAO(
				"orderSettingDAOImplementer");
		BeanUtils.copyProperties(orderSetting, parameters);
		dao.setOrderSetting(orderSetting);
		dao.loadOrderSettingFromID();
		return dao.getOrderSetting();
	}

	public void editOrderSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Editing order setting: " + code);
		}
		OrderSetting orderSetting = (OrderSetting) getBusinessObjectFactory()
				.createObject("orderSettingImplementer");
		BeanUtils.copyProperties(orderSetting, parameters);
		Order order = (Order) getBusinessObjectFactory().createObject(
				"orderImplementer");
		order.setOrderID(orderSetting.getOrderID());
		orderSetting.setOrder(order);
		OrderSettingDAO dao = (OrderSettingDAO) getDaoFactory().createDAO(
				"orderSettingDAOImplementer");
		dao.setOrderSetting(orderSetting);
		dao.updateOrderSetting(true);
		return;
	}

	public void deleteOrderSetting(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Deleting order setting: " + code);
		}
		OrderSetting orderSetting = (OrderSetting) getBusinessObjectFactory()
				.createObject("orderSettingImplementer");
		BeanUtils.copyProperties(orderSetting, parameters);
		OrderSettingDAO dao = (OrderSettingDAO) getDaoFactory().createDAO(
				"orderSettingDAOImplementer");
		dao.setOrderSetting(orderSetting);
		dao.deleteOrderSetting(true);
		return;
	}

	public OrderSetting findOrderSettingByCode(Order order, String code) {
		OrderSetting orderSetting = null;
		if (order != null) {
			Map orderSettings = order.getOrderSettings();
			if (orderSettings != null && orderSettings.containsKey(code)) {
				orderSetting = (OrderSetting) orderSettings.get(code);
			}
		}
		return orderSetting;
	}

	public void addOrUpdateOrderSetting(Order order, OrderSetting orderSetting,
			boolean commitOrder) throws Exception {
		if (order != null && orderSetting != null
				&& StringUtils.isNotBlank(orderSetting.getCode())) {
			Map props = PropertyUtils.describe(orderSetting);
			// Remove order info in case its in there, we use the sent in order
			// instead
			props.remove("orderID");
			props.remove("order");
			props.remove("orderSettingID");
			Map orderSettings = order.getOrderSettings();
			if (orderSettings != null
					&& orderSettings.containsKey(orderSetting.getCode())) {
				OrderSetting os = (OrderSetting) orderSettings.get(orderSetting
						.getCode());
				PropertyUtils.copyProperties(os, props);
			} else {
				// orderSetting.setOrderID(order.getOrderID());
				orderSetting.setOrder(order);
				if (orderSettings == null) {
					Map newOrderSettings = new HashMap();
					newOrderSettings.put(orderSetting.getCode(), orderSetting);
					order.setOrderSettings(newOrderSettings);
				} else {
					orderSettings.put(orderSetting.getCode(), orderSetting);
				}
			}
			if (commitOrder) {
				OrderDAO orderDAO = (OrderDAO) getDaoFactory().createDAO(
						"orderDAOImplementer");
				orderDAO.setOrder(order);
				orderDAO.updateOrder();
			}
		}
	}
}
