package com.softslate.commerce.businessobjects.order;

import java.io.Serializable;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing an order setting within the system.
 * <code>OrderSettingBean</code> is the default implementation of the
 * <code>OrderSetting</code> interface for the application.
 * <p>
 * The database stores order settings in the <code>sscOrderSetting</code>
 * table, and this object can be used to represent a single row of that
 * table. The Hibernate configuration file
 * <code>OrderSettingBean.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudOrderSetting")
public class OrderSettingBean extends BaseBusinessObject implements
		OrderSetting, Serializable {

	private static final long serialVersionUID = -5702986956134527509L;

	private int orderSettingID = 0;

	public int getOrderSettingID() {
		return orderSettingID;
	}

	public void setOrderSettingID(int orderSettingID) {
		this.orderSettingID = orderSettingID;
	}

	private Order order = null;

	public Order getOrder() {
		return order;
	}

	public void setOrder(Order order) {
		this.order = order;
	}

	private int orderID = 0;

	private String code = null;

	private String name = null;

	private String description = null;

	private String type = null;

	private String valueType = null;

	private Integer integerValue;

	private String smallValue = null;

	private String mediumValue = null;

	private String textValue = null;

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public Integer getIntegerValue() {
		return integerValue;
	}

	public void setIntegerValue(Integer integerValue) {
		this.integerValue = integerValue;
	}

	public String getMediumValue() {
		return mediumValue;
	}

	public void setMediumValue(String mediumValue) {
		this.mediumValue = mediumValue;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getOrderID() {
		return orderID;
	}

	public void setOrderID(int orderID) {
		this.orderID = orderID;
	}

	public String getSmallValue() {
		return smallValue;
	}

	public void setSmallValue(String smallValue) {
		this.smallValue = smallValue;
	}

	public String getTextValue() {
		return textValue;
	}

	public void setTextValue(String textValue) {
		this.textValue = textValue;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getValueType() {
		return valueType;
	}

	public void setValueType(String valueType) {
		this.valueType = valueType;
	}

	public String getValue() {
		if (getValueType().equals("smallValue")) {
			return getSmallValue();
		} else if (getValueType().equals("mediumValue")) {
			return getMediumValue();
		} else if (getValueType().equals("textValue")) {
			return getTextValue();
		} else if (getValueType().equals("integerValue")
				|| getValueType().equals("booleanValue")) {
			return Integer.toString(getIntegerValue());
		}
		return null;
	}
}
