package com.softslate.commerce.businessobjects.order;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;
import com.softslate.commerce.businessobjects.shipping.ShippingRule;

/**
 * Interface representing an order shipping rule within the system. An order shipping rule
 * can be thought of as a
 * {@link com.softslate.commerce.businessobjects.shipping.ShippingRule} that has been
 * applied to a user's order, either on the level of the order as whole or on
 * the level of a single order item.
 * <p>
 * When the application needs to create an instance that implements
 * <code>OrderShippingRule</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "orderShippingRuleImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "orderShippingRuleImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderShippingRuleBean}.
 * 
 * @author Jason McWilliams
 * 
 * @see com.softslate.commerce.businessobjects.shipping.ShippingRule
 * 
 */
@Audited
@AuditTable("AudOrderShippingRule")
public interface OrderShippingRule extends BusinessObject {
	
	abstract int getOrderShippingRuleID();
	abstract void setOrderShippingRuleID(int orderShippingRuleID);
	
	abstract int getOrderID();
	abstract void setOrderID(int orderID);
	
	abstract Integer getOrderItemID();
	abstract void setOrderItemID(Integer orderItemID);
	
	abstract Double getTotal();
	abstract void setTotal(Double total);
	
	abstract Integer getShippingRuleID();
	abstract void setShippingRuleID(Integer shippingRuleID);
	
	abstract String getCode();
	abstract void setCode(String code);
	
	abstract String getName();
	abstract void setName(String name);

	abstract String getRuleType();
	abstract void setRuleType(String ruleType);
	
	abstract Double getFloor();
	abstract void setFloor(Double floor);
	
	abstract Double getCeiling();
	abstract void setCeiling(Double ceiling);
	
	abstract String getDiscountType();
	abstract void setDiscountType(String discountType);
	
	abstract Double getDiscount();
	abstract void setDiscount(Double discount);
	
	abstract String getPkgDescription();
	abstract void setPkgDescription(String pkgDescription);
	
	abstract Double getPkgWeight();
	abstract void setPkgWeight(Double pkgWeight);
	
	abstract Double getPkgLength();
	abstract void setPkgLength(Double pkgLength);
	
	abstract Double getPkgHeight();
	abstract void setPkgHeight(Double pkgHeight);
	
	abstract Double getPkgWidth();
	abstract void setPkgWidth(Double pkgWidth);
	
	abstract String getLimitMethods();
	abstract void setLimitMethods(String limitMethods);
	
	abstract String getLimitType();
	abstract void setLimitType(String limitType);
	
	abstract String getExtra1();
	abstract void setExtra1(String extra1);

	abstract String getExtra2();
	abstract void setExtra2(String extra2);

	abstract String getExtra3();
	abstract void setExtra3(String extra3);
	
	abstract String getPkgExtra1();
	abstract void setPkgExtra1(String pkgExtra1);
	
	abstract String getPkgExtra2();
	abstract void setPkgExtra2(String pkgExtra2);
	
	abstract String getPkgExtra3();
	abstract void setPkgExtra3(String pkgExtra3);
	
	abstract String getPkgExtra4();
	abstract void setPkgExtra4(String pkgExtra4);
	
	abstract String getPkgExtra5();
	abstract void setPkgExtra5(String pkgExtra5);
	
	abstract Double getSubtotal();
	abstract void setSubtotal(Double subtotal);
	
	abstract ShippingRule getShippingRule();
	abstract void setShippingRule(ShippingRule shippingRule);

	abstract Order getOrder();
	abstract void setOrder(Order order);

	abstract OrderItem getOrderItem();
	abstract void setOrderItem(OrderItem orderItem);
	
	abstract String getCountries();
	abstract void setCountries(String countries);
	
	abstract String getStates();
	abstract void setStates(String states);


}
