package com.softslate.commerce.businessobjects.order;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a state within the system.
 * <p>
 * When the application needs to create an instance that implements
 * <code>sscState</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the "stateImplementer"
 * setting in the <code>appComponents.properties</code> file.
 * <p>
 * The default "stateImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.StateBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudState")
public interface State extends BusinessObject {
	abstract String getCode();

	abstract void setCode(String code);

	abstract int getCountryID();

	abstract void setCountryID(int countryID);

	abstract boolean getIsActive();

	abstract void setIsActive(boolean isActive);

	abstract String getName();

	abstract void setName(String name);

	abstract int getStateOrder();

	abstract void setStateOrder(int stateOrder);

	abstract int getStateID();

	abstract void setStateID(int stateID);

	abstract Country getCountry();

	abstract void setCountry(Country country);
}
