package com.softslate.commerce.businessobjects.payment;

import java.io.Serializable;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;
import com.softslate.commerce.businessobjects.order.Order;

/**
 * Object representing a payment within the system. <code>PaymentBean</code>
 * is the default implementation of the <code>Payment</code> interface for the
 * application.
 * <p>
 * The database stores payments in the <code>sscPayment</code> table, and this
 * interface can be used to represent a single row of that table. The Hibernate
 * configuration file <code>PaymentBean.hbm.xml</code> maps that table's
 * columns to the properties in this object.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudPayment")
public class PaymentBean extends BaseBusinessObject implements Payment,
		Serializable {

	private static final long serialVersionUID = 6172928043433996107L;

	private int paymentID = 0;

	private String status = null;

	private int orderID = 0;

	private String processorClassName = null;

	private String valueFieldLabels = null;

	private String creditCardName = null;

	private String creditCardType = null;

	private String creditCardDate = null;

	private String decryptedNumber = null;

	private String creditCardNumber = null;

	private String creditCardNumberDisplay = null;

	private String value1 = null;

	private String value2 = null;

	private String value3 = null;

	private String value4 = null;

	private String value5 = null;

	private String value6 = null;

	private String value7 = null;

	private String value8 = null;

	private String value9 = null;

	private String created = null;

	private Double amount;

	private String extra1 = null;

	private String extra2 = null;

	private String extra3 = null;

	private String paymentProfileID = null;

	private Order order = null;

	public String getDecryptedNumber() {
		return decryptedNumber;
	}

	public void setDecryptedNumber(String decryptedNumber) {
		this.decryptedNumber = decryptedNumber;
	}

	public Order getOrder() {
		return order;
	}

	public void setOrder(Order order) {
		this.order = order;
	}

	public int getPaymentID() {
		return paymentID;
	}

	public void setPaymentID(int paymentID) {
		this.paymentID = paymentID;
	}

	public String getCreditCardDate() {
		return creditCardDate;
	}

	public void setCreditCardDate(String creditCardDate) {
		this.creditCardDate = creditCardDate;
	}

	public String getCreditCardName() {
		return creditCardName;
	}

	public void setCreditCardName(String creditCardName) {
		this.creditCardName = creditCardName;
	}

	public String getCreditCardNumber() {
		return creditCardNumber;
	}

	public void setCreditCardNumber(String creditCardNumber) {
		this.creditCardNumber = creditCardNumber;
	}

	public String getCreditCardType() {
		return creditCardType;
	}

	public void setCreditCardType(String creditCardType) {
		this.creditCardType = creditCardType;
	}

	public int getOrderID() {
		return orderID;
	}

	public void setOrderID(int orderID) {
		this.orderID = orderID;
	}

	public String getCreditCardNumberDisplay() {
		return creditCardNumberDisplay;
	}

	public void setCreditCardNumberDisplay(String creditCardNumberDisplay) {
		this.creditCardNumberDisplay = creditCardNumberDisplay;
	}

	public String getProcessorClassName() {
		return processorClassName;
	}

	public void setProcessorClassName(String processorClassName) {
		this.processorClassName = processorClassName;
	}

	public String getStatus() {
		return status;
	}

	public void setStatus(String status) {
		this.status = status;
	}

	public String getValue1() {
		return value1;
	}

	public void setValue1(String value1) {
		this.value1 = value1;
	}

	public String getValue2() {
		return value2;
	}

	public void setValue2(String value2) {
		this.value2 = value2;
	}

	public String getValue3() {
		return value3;
	}

	public void setValue3(String value3) {
		this.value3 = value3;
	}

	public String getValue4() {
		return value4;
	}

	public void setValue4(String value4) {
		this.value4 = value4;
	}

	public String getValue5() {
		return value5;
	}

	public void setValue5(String value5) {
		this.value5 = value5;
	}

	public String getValue6() {
		return value6;
	}

	public void setValue6(String value6) {
		this.value6 = value6;
	}

	public String getValue7() {
		return value7;
	}

	public void setValue7(String value7) {
		this.value7 = value7;
	}

	public String getValue8() {
		return value8;
	}

	public void setValue8(String value8) {
		this.value8 = value8;
	}

	public String getValue9() {
		return value9;
	}

	public void setValue9(String value9) {
		this.value9 = value9;
	}

	public String getCreated() {
		return created;
	}

	public void setCreated(String created) {
		this.created = created;
	}

	public Double getAmount() {
		return amount;
	}

	public void setAmount(Double amount) {
		this.amount = amount;
	}

	public String getValueFieldLabels() {
		return valueFieldLabels;
	}

	public void setValueFieldLabels(String valueFieldLabels) {
		this.valueFieldLabels = valueFieldLabels;
	}

	public String getExtra1() {
		return extra1;
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	public String getExtra2() {
		return extra2;
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	public String getExtra3() {
		return extra3;
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	public String getPaymentProfileID() {
		return paymentProfileID;
	}

	public void setPaymentProfileID(String paymentProfileID) {
		this.paymentProfileID = paymentProfileID;
	}

}
