package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a product setting within the system.
 * <code>ProductSettingBean</code> is the default implementation of the
 * <code>ProductSetting</code> interface for the application.
 * <p>
 * The database stores product settings in the <code>sscProductSetting</code>
 * table, and this interface can be used to represent a single row of that
 * table. The Hibernate configuration file
 * <code>ProductSettingBean.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudProductSetting")
public class ProductSettingBean extends BaseBusinessObject implements
		ProductSetting, Serializable {

	private static final long serialVersionUID = -5702986956134527509L;

	private int productSettingID = 0;

	public int getProductSettingID() {
		return productSettingID;
	}

	public void setProductSettingID(int productSettingID) {
		this.productSettingID = productSettingID;
	}

	private Product product = null;

	public Product getProduct() {
		return product;
	}

	public void setProduct(Product product) {
		this.product = product;
	}

	private int productID = 0;

	private String code = null;

	private String name = null;

	private String description = null;

	private String type = null;

	private String valueType = null;

	private int integerValue;

	private String smallValue = null;

	private String mediumValue = null;

	private String textValue = null;

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public int getIntegerValue() {
		return integerValue;
	}

	public void setIntegerValue(int integerValue) {
		this.integerValue = integerValue;
	}

	public String getMediumValue() {
		return mediumValue;
	}

	public void setMediumValue(String mediumValue) {
		this.mediumValue = mediumValue;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getProductID() {
		return productID;
	}

	public void setProductID(int productID) {
		this.productID = productID;
	}

	public String getSmallValue() {
		return smallValue;
	}

	public void setSmallValue(String smallValue) {
		this.smallValue = smallValue;
	}

	public String getTextValue() {
		return textValue;
	}

	public void setTextValue(String textValue) {
		this.textValue = textValue;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getValueType() {
		return valueType;
	}

	public void setValueType(String valueType) {
		this.valueType = valueType;
	}

	public String getValue() {
		if (getValueType().equals("smallValue")) {
			return getSmallValue();
		} else if (getValueType().equals("mediumValue")) {
			return getMediumValue();
		} else if (getValueType().equals("textValue")) {
			return getTextValue();
		} else if (getValueType().equals("integerValue")
				|| getValueType().equals("booleanValue")) {
			return Integer.toString(getIntegerValue());
		}
		return null;
	}
}
