package com.softslate.commerce.businessobjects.shipping;

import java.io.Serializable;
import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing a shipping method within the system.
 * <code>ShippingMethodBean</code> is the default implementation of the
 * <code>ShippingMethod</code> interface for the application.
 * <p>
 * The database stores shipping methods in the <code>sscShippingMethod</code>
 * table, and this interface can be used to represent a single row of that
 * table. The Hibernate configuration file
 * <code>ShippingMethodBean.hbm.xml</code> maps that table's columns to the
 * properties in this object.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudShippingMethod")
public class ShippingMethodBean extends BaseBusinessObject implements
		ShippingMethod, Serializable {

	private static final long serialVersionUID = 1394945028842260924L;

	private int shippingMethodID = 0;

	private String code = null;

	private String name = null;

	private Double minimum = null;

	private Double base = null;

	private String methodType = null;

	private boolean isActive = true;

	private Integer numberDaysToShip = null;

	private Integer methodOrder = null;

	private Collection shippingRates = null;

	public Collection getShippingRates() {
		return shippingRates;
	}

	public void setShippingRates(Collection shippingRates) {
		this.shippingRates = shippingRates;
	}

	public int getShippingMethodID() {
		return shippingMethodID;
	}

	public void setShippingMethodID(int shippingMethodID) {
		this.shippingMethodID = shippingMethodID;
	}

	public Double getBase() {
		return base;
	}

	public void setBase(Double base) {
		this.base = base;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public boolean getIsActive() {
		return isActive;
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	public String getMethodType() {
		return methodType;
	}

	public void setMethodType(String methodType) {
		this.methodType = methodType;
	}

	public Double getMinimum() {
		return minimum;
	}

	public void setMinimum(Double minimum) {
		this.minimum = minimum;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Integer getNumberDaysToShip() {
		return numberDaysToShip;
	}

	public void setNumberDaysToShip(Integer numberDaysToShip) {
		this.numberDaysToShip = numberDaysToShip;
	}

	public Integer getMethodOrder() {
		return methodOrder;
	}

	public void setMethodOrder(Integer methodOrder) {
		this.methodOrder = methodOrder;
	}
	
}
