package com.softslate.commerce.businessobjects.tax;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.order.Country;
import com.softslate.commerce.businessobjects.order.State;
import com.softslate.commerce.daos.tax.BasicTaxRateDAO;
import com.softslate.commerce.daos.tax.BasicTaxRateGatewayDAO;

/**
 * Processes business logic having to do with basic tax rates. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting basic tax rates.
 * <p>
 * <code>BasicTaxRateProcessor</code> is the default implementation of the
 * <code>TaxRateProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 */
public class BasicTaxRateProcessor extends BaseBusinessProcessor implements
        TaxRateProcessor {

    static Log log = LogFactory.getLog(BasicTaxRateProcessor.class);

    public Map loadBasicTaxRatesAndCount(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading basic tax rates and count");
        BasicTaxRateGatewayDAO taxRateDAO =  getDaoFactory()
                .createDAO(BasicTaxRateGatewayDAO.class);
        return taxRateDAO.loadBasicTaxRatesAndCount(parameters);
    }

    public Map addTaxRate(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Adding basic tax rate");
		Map results = new HashMap();
        BasicTaxRate basicTaxRate =  getBusinessObjectFactory()
                .createObject(BasicTaxRate.class);
        BeanUtils.copyProperties(basicTaxRate, parameters);
        Country country = (Country) getBusinessObjectFactory().createObject(
                "countryImplementer");
        country.setCountryID(basicTaxRate.getCountryID().intValue());
        basicTaxRate.setCountry(country);
        if (basicTaxRate.getStateID() != null
                && basicTaxRate.getStateID().intValue() != 0) {
            State state = (State) getBusinessObjectFactory().createObject(
                    "stateImplementer");
            state.setStateID(basicTaxRate.getStateID().intValue());
            basicTaxRate.setState(state);
        }
        BasicTaxRateDAO taxRateDAO =  getDaoFactory()
                .createDAO(BasicTaxRateDAO.class);
        taxRateDAO.setBasicTaxRate(basicTaxRate);
        taxRateDAO.insertBasicTaxRate(true);
		results.put("newRecord", taxRateDAO.getBasicTaxRate());
		return results;
    }

    public BasicTaxRate loadBasicTaxRateFromID(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Loading basic tax rate from id");
        BasicTaxRate basicTaxRate =  getBusinessObjectFactory()
                .createObject(BasicTaxRate.class);
        BasicTaxRateDAO taxRateDAO =  getDaoFactory()
                .createDAO(BasicTaxRateDAO.class);
        BeanUtils.copyProperties(basicTaxRate, parameters);
        taxRateDAO.setBasicTaxRate(basicTaxRate);
        taxRateDAO.loadBasicTaxRateFromID();
		return taxRateDAO.getBasicTaxRate();
    }

    public Map editTaxRate(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing basic tax rate");
		Map results = new HashMap();

		// Load the tax rate record
		BasicTaxRate basicTaxRate = loadBasicTaxRateFromID(parameters);
        BeanUtils.copyProperties(basicTaxRate, parameters);

		// Update the tax rate record
        Country country = (Country) getBusinessObjectFactory().createObject(
                "countryImplementer");
        country.setCountryID(basicTaxRate.getCountryID().intValue());
        basicTaxRate.setCountry(country);
        if (basicTaxRate.getStateID() != null
                && basicTaxRate.getStateID().intValue() != 0) {
            State state = (State) getBusinessObjectFactory().createObject(
                    "stateImplementer");
            state.setStateID(basicTaxRate.getStateID().intValue());
            basicTaxRate.setState(state);
        }
        BasicTaxRateDAO taxRateDAO =  getDaoFactory()
                .createDAO(BasicTaxRateDAO.class);
        taxRateDAO.setBasicTaxRate(basicTaxRate);
        taxRateDAO.updateBasicTaxRate(true);
		results.put("updatedRecord", taxRateDAO.getBasicTaxRate());
		return results;
    }

    public Map editDeleteTaxRates(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Editing/deleting multiple basic tax rates");
		String[] idValues = (String[]) parameters.get("idValues");
		String[] displayFields = (String[]) parameters.get("displayFields");
		Country[] country = new Country[idValues.length];
		for (int i = 0; i < idValues.length; i++) {
			country[i] = null;
			for (int j = 0; j < displayFields.length; j++) {
				String displayField = displayFields[j];
				if (displayField.equals("countryID")
						|| displayField.equals("country")) {
					displayFields[j] = "country";
					String[] fieldValues = (String[]) parameters
							.get("countryID");
					if (fieldValues != null && fieldValues[i] != null
							&& !fieldValues[i].equals("0")) {
						if (log.isDebugEnabled())
							log.debug("Creating country of "
									+ fieldValues[i] + " for id: "
									+ idValues[i]);
						Country object =  getBusinessObjectFactory()
								.createObject(Country.class);
						object.setCountryID((new Integer(fieldValues[i]))
								.intValue());
						country[i] = object;
					}
				}
			}
		}
		parameters.put("country", country);
		State[] state = new State[idValues.length];
		for (int i = 0; i < idValues.length; i++) {
			state[i] = null;
			for (int j = 0; j < displayFields.length; j++) {
				String displayField = displayFields[j];
				if (displayField.equals("stateID")
						|| displayField.equals("state")) {
					displayFields[j] = "state";
					String[] fieldValues = (String[]) parameters
							.get("stateID");
					if (fieldValues != null && fieldValues[i] != null
							&& !fieldValues[i].equals("0")) {
						if (log.isDebugEnabled())
							log.debug("Creating state of "
									+ fieldValues[i] + " for id: "
									+ idValues[i]);
						State object =  getBusinessObjectFactory()
								.createObject(State.class);
						object.setStateID((new Integer(fieldValues[i]))
								.intValue());
						state[i] = object;
					}
				}
			}
		}
		parameters.put("state", state);
        BasicTaxRateGatewayDAO taxRateDAO =  getDaoFactory()
                .createDAO(BasicTaxRateGatewayDAO.class);
		Map results = taxRateDAO.editDeleteTaxRates(parameters, getBusinessObjectFactory());
		return results;
    }

    public void deleteTaxRate(Map parameters) throws Exception {
        if (log.isInfoEnabled() && getUser().getAdministrator() != null)
            log.info(getUser().getAdministrator().getUserName()
                    + ": Deleting basic tax rate");
        BasicTaxRate basicTaxRate =  getBusinessObjectFactory()
                .createObject(BasicTaxRate.class);
        BeanUtils.copyProperties(basicTaxRate, parameters);
        BasicTaxRateDAO taxRateDAO =  getDaoFactory()
                .createDAO(BasicTaxRateDAO.class);
        taxRateDAO.setBasicTaxRate(basicTaxRate);
        taxRateDAO.deleteBasicTaxRate(true);
        return;
    }

	public int loadMaxBasicTaxRateID() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			log.info(getUser().getAdministrator().getUserName() + ": Loading max BasicTaxRate ID.");
		}
		BasicTaxRateGatewayDAO gatewayDAO = getDaoFactory().createDAO(BasicTaxRateGatewayDAO.class);
		return gatewayDAO.loadMaxBasicTaxRateID();
	}
}
