package com.softslate.commerce.customer.customer;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class that corresponds to the store's "Your Account" link. Used
 * for requests to "/Account.do".
 * <p>
 * The <code>execute</code> method of this class checks to see if the user is
 * currently logged in. If so, it returns the <code>ActionForward</code> named
 * "accountHome". If not, it returns the <code>ActionForward</code> named
 * "accountLogin".
 * 
 * @author David Tobey
 */
public class AccountAction extends BaseAction {
	static Log log = LogFactory.getLog(AccountAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = new BaseForm();
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);
		// Check for catalog mode
		String catalogMode = (String) baseForm.getSettings().getValue(
				"catalogMode");
		if (catalogMode.equals("1")) {
			return mapping.findForward("catalogMode");
		}
		synchronized (baseForm.getRequest().getSession()) {
			// If the customer object in the user object is null, then we have
			// not logged in.
			if (log.isDebugEnabled())
				log.debug("Checking if user is a logged in customer.");
			if (baseForm.getUser().getCustomer() == null) {
				if (log.isDebugEnabled())
					log.debug("User is not logged in.");
				return mapping.findForward("accountLogin");
			}
			// Forward to the account home
			return mapping.findForward("accountHome");
		}
	}
}
