package com.softslate.commerce.customer.customer;

import java.util.Map;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class used for requests to save a cart by a customer. Used for
 * requests to "/SaveCart.do".
 * 
 * @author David Tobey
 * 
 */
public class SaveCartAction extends BaseAction {
	static Log log = LogFactory.getLog(SaveCartAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = new BaseForm();
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);
		synchronized (baseForm.getRequest().getSession()) {

			// Preserve the original request so the customer is redirected
			// back here after logging in
			request.setAttribute("originalRequest", request.getRequestURI());

			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.saveCartMustBeLoggedIn"));
				this.saveMessages(request, baseForm.getMessages());
				if (log.isInfoEnabled() && !baseForm.getMessages().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getMessages().toString());
				return mapping.findForward("notLoggedIn");
			}

			// Check the global allowSavedCarts flag
			if (!baseForm.getSettings().getValue("allowSavedCarts").equals("1")) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.cartsMayNotBeSaved"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): " + baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}

			// Clear the tax and taxable subtotal if shipping and taxes are not
			// processed. This will help us know the state of the order when it
			// is picked up.
			if (baseForm.getRequest().getSession()
					.getAttribute("shippingAndTaxesProcessed") == null
					|| baseForm.getRequest().getSession()
							.getAttribute("shippingAndTaxesProcessed")
							.equals("false")) {
				baseForm.getUser().getOrder().setTax(null);
				baseForm.getUser().getOrder().setTaxableSubtotal(null);
			} else {
				baseForm.getRequest().getSession()
						.setAttribute("shippingAndTaxesProcessed", "false");
			}

			// Save the cart
			CustomerProcessor processor = (CustomerProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"customerProcessorImplementer");
			Map result = processor.saveCart(null);
			if (result != null && result.get("resultCode") != null
					&& !result.get("resultCode").equals("0")) {
				String resultCode = (String) result.get("resultCode");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage(resultCode));
				saveErrors(baseForm.getRequest(),
						(ActionMessages) baseForm.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}

			// Nullify the session's cart to start over.
			baseForm.getUser().setOrder(null);

			// Delete the savedCartToken cookie so the cart isn't reloaded
			// right away again
			Cookie[] cookies = request.getCookies();
			if (cookies != null) {
				for (int i = 0; i < cookies.length; i++) {
					if (cookies[i] != null && cookies[i].getName() != null
							&& cookies[i].getName().equals("sscsctoken")) {
						Cookie savedCartCookie = cookies[i];
						savedCartCookie.setMaxAge(0);
						response.addCookie(savedCartCookie);
					}
				}
			}

			// Tell the CustomerRequestProcessor to skip loading the saved
			// cart as we forward to another action.
			request.setAttribute("skipLoadSavedCart", "true");

			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.cartSaved"));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());

			return mapping.findForward("success");
		}
	}
}
