package com.softslate.commerce.customer.order;

import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.order.CartProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class for submissions to add items to the current user's cart.
 * Used for requests to "/CartAdd.do".
 * <p>
 * The <code>execute</code> method of this class invokes
 * {@link com.softslate.commerce.businessobjects.order.CartProcessor#processAddItems(Map)}
 * to process the add cart request and insert the items into the database. It
 * then performs processing of the results returned from that method, including
 * sending low stock email messages.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.customer.order.CartAddForm
 * @see com.softslate.commerce.businessobjects.order.CartProcessor
 */
public class CartAddAction extends BaseAction {

	static Log log = LogFactory.getLog(CartAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = (BaseForm) form;
		baseForm.initializeProperties(mapping, request);
		synchronized (request.getSession()) {

			// Clear the shipping and taxes flag.
			baseForm.getRequest().getSession().setAttribute(
					"shippingAndTaxesProcessed", "false");

			// Set the saved cart token as a cookie
			Map parameters = PropertyUtils.describe(baseForm);
			processSavedCartCookie(baseForm, parameters, response);

			// Process the items
			CartProcessor cartProcessor = (CartProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"cartProcessorImplementer");
			Map results = cartProcessor.processAddItems(parameters);
			request.setAttribute("newOrderItems", results.get("newOrderItems"));
			request.setAttribute("productsAdded", results.get("productsAdded"));

			// Process inventory actions
			String forward = processResults(results, baseForm);

			// Load new estimated shipping costs
			loadEstimatedShippingOptions(baseForm);

			return mapping.findForward(forward);

		}
	}

	public void processSavedCartCookie(BaseForm baseForm, Map parameters,
			HttpServletResponse response) {
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		actionUtils.processSavedCartCookie(baseForm, parameters, response);
	}

	public String processResults(Map results, BaseForm baseForm) {
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		return actionUtils.processCartAddResults(results, baseForm);
	}

	public void addInventoryMessages(Map results, BaseForm baseForm) {
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		actionUtils.addInventoryMessages(results, baseForm);
	}

	public Collection loadEstimatedShippingOptions(BaseForm baseForm) {
		ActionUtils actionUtils = (ActionUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"actionUtilsImplementer");
		return actionUtils.loadEstimatedShippingOptions(baseForm);
	}

}
