package com.softslate.commerce.customer.order;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerAddress;
import com.softslate.commerce.businessobjects.customer.CustomerAddressProcessor;

/**
 * Struts form class for submissions to record address information during
 * checkout. Used for requests to "/CheckoutAddresses.do".
 * <p>
 * <code>CheckoutAddressesForm</code> is a subclass of
 * {@link com.softslate.commerce.customer.order.AddressesForm}, and it inherits
 * its billing and delivery address properties.
 * <p>
 * The <code>validate</code> method of this class runs the
 * <code>validate</code> method of <code>AddressesForm</code> to make sure
 * all the required billing and delivery fields are present.
 * 
 * @author David Tobey
 */
public class CheckoutAddressesForm extends AddressesForm {

	private static final long serialVersionUID = 405356478771279576L;

	static Log log = LogFactory.getLog(CheckoutAddressesForm.class);

	private boolean isSubmitted = false;

	public boolean isSubmitted() {
		return isSubmitted;
	}

	public void setIsSubmitted(boolean isSubmitted) {
		this.isSubmitted = isSubmitted;
	}

	/**
	 * If "true" this property indicates that if the form is being submitted by
	 * a logged-in customer, the customer's account information should be
	 * updated.
	 */
	private String updateCustomer = "true";

	public String getUpdateCustomer() {
		return updateCustomer;
	}

	public void setUpdateCustomer(String updateCustomer) {
		this.updateCustomer = updateCustomer;
	}

	private String emailerOptInApproval = "Yes";

	public String getEmailerOptInApproval() {
		return emailerOptInApproval;
	}

	public void setEmailerOptInApproval(String emailerOptInApproval) {
		this.emailerOptInApproval = emailerOptInApproval;
	}

	private String addToAddressBook = null;

	public String getAddToAddressBook() {
		return addToAddressBook;
	}

	public void setAddToAddressBook(String addToAddressBook) {
		this.addToAddressBook = addToAddressBook;
	}

	private String addToBillingAddressBook = null;

	public String getAddToBillingAddressBook() {
		return addToBillingAddressBook;
	}

	public void setAddToBillingAddressBook(String addToBillingAddressBook) {
		this.addToBillingAddressBook = addToBillingAddressBook;
	}

	private String customerAddressID = null;

	public String getCustomerAddressID() {
		return customerAddressID;
	}

	public void setCustomerAddressID(String customerAddressID) {
		this.customerAddressID = customerAddressID;
	}

	private String customerBillingAddressID = null;

	public String getCustomerBillingAddressID() {
		return customerBillingAddressID;
	}

	public void setCustomerBillingAddressID(String customerBillingAddressID) {
		this.customerBillingAddressID = customerBillingAddressID;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");

		// On the checkout screen, do not call super.validate, call all here to
		// deal with address book. We have to deal with the address book after
		// validation the billing and before validating the delivery
		initializeProperties(mapping, request);
		String allowAddressBook = (String) getSettings().getValue("allowAddressBook");
		if ("1".equals(allowAddressBook) && getUser() != null && getUser().getCustomer() != null
				&& StringUtils.isNotBlank(getCustomerBillingAddressID())) {
			int customerBillingAddressID = 0;
			try {
				customerBillingAddressID = Integer.parseInt(getCustomerBillingAddressID());
			} catch (Exception nfe) {
				// swallow
			}
			try {
				CustomerAddressProcessor customerAddressProcessor =  getBusinessObjectFactory()
						.createObject(CustomerAddressProcessor.class);
				CustomerAddress customerAddress = customerAddressProcessor.findCustomerAddress(getUser().getCustomer()
						.getCustomerAddressCollection(), customerBillingAddressID);
				if (customerAddress == null) {
					customerAddress = (CustomerAddress) getBusinessObjectFactory().createObject(
							"customerAddressImplementer");
				}
				Map props = customerAddressProcessor.describeDeliveryOrBillingForAddressBook(
						PropertyUtils.describe(customerAddress), "billing", false);
				BeanUtils.populate(this, props);

			} catch (Exception ex) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.general"));
			}
		}
		validateBillingFields();
		copyBillingToDelivery();

		// Validate the address fields
		if ("1".equals(allowAddressBook) && getUser() != null
				&& getUser().getCustomer() != null
				&& StringUtils.isNotBlank(getCustomerAddressID())) {
			int customerAddressID = 0;
			try {
				customerAddressID = Integer.parseInt(getCustomerAddressID());
			} catch (Exception nfe) {
				// swallow
			}
			try {
				CustomerAddressProcessor customerAddressProcessor =  getBusinessObjectFactory()
						.createObject(CustomerAddressProcessor.class);
				CustomerAddress customerAddress = customerAddressProcessor
						.findCustomerAddress(getUser().getCustomer()
								.getCustomerAddressCollection(),
								customerAddressID);
				if (customerAddress == null) {
					customerAddress =  getBusinessObjectFactory()
							.createObject(CustomerAddress.class);
				}
				Map props = customerAddressProcessor.describeDeliveryOrBillingForAddressBook(
						PropertyUtils.describe(customerAddress), "delivery", false);
				BeanUtils.populate(this, props);
			} catch (Exception ex) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.general"));
			}
		}

		// Set the emailOptInApproval property to no if it was not actually
		// submitted.
		if (request.getParameter("emailerOptInApproval") == null) {
			setEmailerOptInApproval("No");
		}

		validateDeliveryFields();

		// Mark the form as being submitted
		setIsSubmitted(true);

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}
}
