package com.softslate.commerce.customer.order;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts action class for submissions to continue checking out without logging
 * in or registering for an account. Used for requests to
 * "/CheckoutDeclineLogin.do".
 * <p>
 * The <code>execute</code> method of this class runs
 * {@link com.softslate.commerce.customer.core.ActionUtils#findCheckoutScreen(BaseForm)}
 * to verifiy that the user is allowed to continue checkout without logging in.
 * It then invokes
 * {@link com.softslate.commerce.customer.order.CheckoutAction#processDeclineLogin(BaseForm)}
 * to process the request.
 * 
 * @author David Tobey
 */
public class CheckoutDeclineLoginAction extends CheckoutAction {
	static Log log = LogFactory.getLog(CheckoutDeclineLoginAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		// Initialize a BaseForm
		BaseForm baseForm = (BaseForm) new BaseForm();
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		synchronized (baseForm.getRequest().getSession()) {
			// Make sure the user is allowed to have this action performed.
			ActionUtils actionUtils = (ActionUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"actionUtilsImplementer");
			String checkoutScreen = actionUtils.findCheckoutScreen(baseForm);
			if (!checkoutScreen.equals("inviteLogin")) {
				if (request.getAttribute("isApiRequest") != null && (Boolean) request.getAttribute("isApiRequest")) {
					baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.checkoutDeclineLogin"));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				}
				return baseForm.getMapping().findForward(checkoutScreen);
			}

			// Process the form.
			return processDeclineLogin(baseForm);
		}
	}
}
