package com.softslate.commerce.customer.payment;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.octo.captcha.module.config.CaptchaModuleConfigHelper;
import com.octo.captcha.module.struts.CaptchaServicePlugin;
import com.octo.captcha.service.CaptchaService;
import com.softslate.commerce.businessobjects.core.RecaptchaProcessor;
import com.softslate.commerce.customer.order.BillingAddressForm;

/**
 * 
 * Address form for adding/editing CIM profile addresses through API.
 * 
 * @author Jason McWilliams
 * 
 */
public class AuthorizeNetBillingAddressForm extends BillingAddressForm {

	private static final long serialVersionUID = 5645613191430737809L;

	static Log log = LogFactory.getLog(AuthorizeNetBillingAddressForm.class);

	public String cimPaymentProfileID = null;

	private String creditCardName = null;

	private String creditCardType = null;

	private String decryptedNumber = null;

	private String creditCardSecurityCode = null;

	private String creditCardExpirationMonth = null;

	private String creditCardExpirationYear = null;

	private String creditCardDate = null;

	private String textFromImage = null;

	private String acceptJsDataValue = null;

	private String acceptJsDataDesc = null;

	public String getCimPaymentProfileID() {
		return cimPaymentProfileID;
	}

	public void setCimPaymentProfileID(String cimPaymentProfileID) {
		this.cimPaymentProfileID = cimPaymentProfileID;
	}

	public String getTextFromImage() {
		return textFromImage;
	}

	public void setTextFromImage(String textFromImage) {
		this.textFromImage = textFromImage;
	}

	public String getCreditCardExpirationMonth() {
		return creditCardExpirationMonth;
	}

	public String getCreditCardExpirationYear() {
		return creditCardExpirationYear;
	}

	public String getDecryptedNumber() {
		return decryptedNumber;
	}

	public void setDecryptedNumber(String decryptedNumber) {
		this.decryptedNumber = decryptedNumber;
	}

	public String getCreditCardSecurityCode() {
		return creditCardSecurityCode;
	}

	public String getCreditCardType() {
		return creditCardType;
	}

	public String getCreditCardDate() {
		return creditCardDate;
	}

	public void setCreditCardDate(String creditCardDate) {
		this.creditCardDate = creditCardDate;
	}

	public void setCreditCardExpirationMonth(String month) {
		creditCardExpirationMonth = month;
	}

	public void setCreditCardExpirationYear(String year) {
		creditCardExpirationYear = year;
	}

	public void setCreditCardSecurityCode(String string) {
		creditCardSecurityCode = string;
	}

	public void setCreditCardType(String string) {
		creditCardType = string;
	}

	public String getCreditCardName() {
		return creditCardName;
	}

	public void setCreditCardName(String creditCardName) {
		this.creditCardName = creditCardName;
	}

	public String getAcceptJsDataValue() {
		return acceptJsDataValue;
	}

	public void setAcceptJsDataValue(String acceptJsDataValue) {
		this.acceptJsDataValue = acceptJsDataValue;
	}

	public String getAcceptJsDataDesc() {
		return acceptJsDataDesc;
	}

	public void setAcceptJsDataDesc(String acceptJsDataDesc) {
		this.acceptJsDataDesc = acceptJsDataDesc;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);

		validateBillingFields();

		if (StringUtils.isNotBlank(getDecryptedNumber())) {
			if (validateCardAndCode()) {
				validateDate();
			}
			validateAcceptJsParams();
			if (validateCardAndCode()) {
				validateSecurityCode();
			}
			validateType();
			if (validateCardAndCode()) {
				validateNumber();
			}
		}
		
		// Ensure that the form submission is coming from the site itself
		if (getCsrfToken() == null || getCsrfToken().length() == 0
				|| !getCsrfToken().equals(getRequest().getSession().getAttribute("csrfToken"))) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.csrfError"));
		}

		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

	protected boolean validateCardAndCode() {
		return !"AcceptJS".equals(getSettings().getValue("authorizeNetIntegrationMethod"));
	}

	protected void validateAcceptJsParams() {
		if ("AcceptJS".equals(getSettings().getValue("authorizeNetIntegrationMethod"))) {
			if (StringUtils.isEmpty(getAcceptJsDataValue()) || StringUtils.isEmpty(getAcceptJsDataDesc())) {
				String message = getMessage("payment", "payment.errorAuthorizeNetAPIProcessorNoNonce", null);
				getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail", message));
			}
		}
	}

	public void validateType() {
		if (getCreditCardType() == null || getCreditCardType().length() == 0) {
			setCreditCardType(formUtils().deriveCardType(getDecryptedNumber()));
		}
	}

	public void validateDate() {
		boolean proceed = true;
		if (getCreditCardExpirationMonth() == null || getCreditCardExpirationMonth().length() == 0) {
			String message = getMessage("order", "paymentForms.ccExpirationMonth", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.required", message));
			proceed = false;
		}
		if (getCreditCardExpirationYear() == null || getCreditCardExpirationYear().length() == 0) {
			String message = getMessage("order", "paymentForms.ccExpirationYear", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.required", message));
			proceed = false;
		}
		if (!proceed) {
			return;
		}
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy", getUser().getLocale());
		String thisYearString = formatter.format(new Date());
		int thisYear = Integer.parseInt(thisYearString);
		if (Integer.parseInt(getCreditCardExpirationYear()) < thisYear) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidExpirationDate"));
		}
		if (Integer.parseInt(getCreditCardExpirationYear()) == thisYear) {
			formatter = new SimpleDateFormat("MM", getUser().getLocale());
			String thisMonthString = formatter.format(new Date());
			int thisMonth = Integer.parseInt(thisMonthString);
			if (Integer.parseInt(getCreditCardExpirationMonth()) < thisMonth) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidExpirationDate"));
			}
		}
		setCreditCardDate(getCreditCardExpirationYear() + getCreditCardExpirationMonth());
	}

	public void validateSecurityCode() {
		if (getCreditCardSecurityCode() != null) {
			setCreditCardSecurityCode(getCreditCardSecurityCode().replaceAll("\\D", ""));
		}
		String isSecurityCodeRequired = null;
		try {
			isSecurityCodeRequired = (String) getSettings().getValue("securityCodeRequired");
		} catch (Exception e) {
		}
		if (isSecurityCodeRequired != null && isSecurityCodeRequired.equals("1")) {
			String message = getMessage("order", "paymentForms.ccSecurityCode", null);
			if (getCreditCardSecurityCode() == null || getCreditCardSecurityCode().length() == 0) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.required", message));
			} else if (getCreditCardSecurityCode() != null && getCreditCardSecurityCode().length() != 3
					&& getCreditCardSecurityCode().length() != 4) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalid", message));
			}
		}
	}

	public void validateNumber() {
		if (getDecryptedNumber() != null) {
			setDecryptedNumber(getDecryptedNumber().replaceAll("\\D", ""));
		}
		if (getDecryptedNumber() == null || getDecryptedNumber().length() == 0) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.cardNumberRequired"));
			return;
		}
		if (!formUtils().isCreditCardValid(getDecryptedNumber())) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidCardNumber"));
			return;
		}
		if (getCreditCardType() != null && getCreditCardType().length() > 0
				&& !formUtils().doCardTypeAndNumberMatch(getCreditCardType(), getDecryptedNumber())) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.cardTypeMismatch"));
			return;
		}

	}

	public void validateJcaptcha() {
		try {
			String useLegacyCaptchaMethod = (String) getSettings().getValue("useLegacyCaptchaMethod");
			String useImageVerificationWithCCInfo = (String) getSettings().getValue(
					"useImageVerificationWithCCInfo");
			if (getErrors().isEmpty() && useImageVerificationWithCCInfo != null
					&& useImageVerificationWithCCInfo.equals("1") && useLegacyCaptchaMethod != null
					&& useLegacyCaptchaMethod.equals("1")) {
				if (getTextFromImage() != null) {
					setTextFromImage(getTextFromImage().toLowerCase());
				}
				CaptchaService service = CaptchaServicePlugin.getInstance().getService();
				String captchaID = CaptchaModuleConfigHelper.getId(getRequest());
				if (captchaID == null || getTextFromImage() == null || getTextFromImage().length() == 0
						|| !service.validateResponseForID(captchaID, getTextFromImage()).booleanValue()) {
					setTextFromImage(null);
					getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidCaptcha"));
				}
			} else if (getErrors().isEmpty() && useImageVerificationWithCCInfo != null
					&& useImageVerificationWithCCInfo.equals("1")) {

				// Google Recaptcha method
				RecaptchaProcessor rp = getBusinessObjectFactory().createObject(RecaptchaProcessor.class);
				Map<String, String> params = new HashMap();
				params.put("response", getRequest().getParameter("g-recaptcha-response"));
				Map response = rp.verifyRecaptcha(params);
				if (response == null || response.get("success") == null
						|| response.get("success").equals(Boolean.FALSE)) {
					getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidReCaptcha"));
				}
			}
		} catch (Exception e) {
			log.error(e.toString(), e);
			setTextFromImage(null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.invalidReCaptcha"));
		}
	}

}
