package com.softslate.commerce.customer.payment;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.payment.AuthorizeNetAPIProcessor;
import com.softslate.commerce.customer.core.BaseAction;

import net.authorize.api.contract.v1.CustomerPaymentProfileMaskedType;

/**
 * 
 * @author Jason McWilliams
 * 
 */
public class AuthorizeNetEditProfileFormAction extends BaseAction {
	static Log log = LogFactory.getLog(AuthorizeNetEditProfileFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");

		AuthorizeNetBillingAddressForm baseForm = (AuthorizeNetBillingAddressForm) form;
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		synchronized (request.getSession()) {

			// Check the Authorize.net settings to make sure the action is allowed.
			String authorizeNetAllowManageProfiles = baseForm.getSettings().getValue("authorizeNetAllowManageProfiles");
			if (request.getAttribute("authorizeNetProfilesOnAndEnabled") == null
					|| !(Boolean) request.getAttribute("authorizeNetProfilesOnAndEnabled")
					|| authorizeNetAllowManageProfiles == null || !authorizeNetAllowManageProfiles.equals("1")) {
				log.info("CIM is not an active payment processor.");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowed"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}
		

			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				log.info("Validation error(s): " + baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}

			// Check Settings
			String authorizeNetIntegrationMethod = (String) baseForm.getSettings()
					.getValue("authorizeNetIntegrationMethod");
			if ("AcceptHostedIframe".equals(authorizeNetIntegrationMethod)) {
				log.info("authorizeNetIntegrationMethod can't be 'AcceptHostedIframe'");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowedAPI"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Retrieve the payment profile from Authorize.net
			Customer customer = baseForm.getUser().getCustomer();
			Order order = baseForm.getUser().getOrder();
			AuthorizeNetAPIProcessor apiProcessor = (AuthorizeNetAPIProcessor) baseForm.getBusinessObjectFactory()
					.createObject("authorizeNetAPIProcessorImplementer");
			CustomerPaymentProfileMaskedType paymentProfile = apiProcessor
					.retrievePaymentProfile(customer.getCimProfileID(), baseForm.getCimPaymentProfileID());
			if (paymentProfile != null) {
				apiProcessor.copyPaymentPaymentProfileToAddressForm(paymentProfile, baseForm);

				// Email is not sent with payments, only profiles, try to find
				// an email to use in case it is required in the form
				if (customer != null && StringUtils.isNotBlank(customer.getEmail1())) {
					baseForm.setBillingEmail1(customer.getEmail1());
				} else if (order != null && StringUtils.isNotBlank(order.getEmail1())) {
					baseForm.setBillingEmail1(order.getEmail1());
				}
				baseForm.getRequest().setAttribute("cimPaymentProfile", paymentProfile);
				baseForm.getRequest().setAttribute("addressForm", baseForm);
				return mapping.findForward("success");
			} else {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesGetProfileError"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}
		}
	}
}
