package com.softslate.commerce.customer.payment;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.payment.AuthorizeNetAPIProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

import net.authorize.api.contract.v1.CustomerPaymentProfileMaskedType;

/**
 * Struts action class used for requests to manage a customer's CIM profiles.
 * Used for requests to "/AuthorizeNetManageProfiles.do".
 * 
 * @author Jason McWilliams
 * 
 */
public class AuthorizeNetManageProfilesAction extends BaseAction {
	static Log log = LogFactory.getLog(AuthorizeNetManageProfilesAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		log.info("Starting action.");
		BaseForm baseForm = (BaseForm) form;
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);
		synchronized (request.getSession()) {

			// Check the Authorize.net settings to make sure the action is allowed.
			String authorizeNetAllowManageProfiles = baseForm.getSettings().getValue("authorizeNetAllowManageProfiles");
			if (request.getAttribute("authorizeNetProfilesOnAndEnabled") == null
					|| !(Boolean) request.getAttribute("authorizeNetProfilesOnAndEnabled")
					|| authorizeNetAllowManageProfiles == null || !authorizeNetAllowManageProfiles.equals("1")) {
				return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesNotAllowed", "fatalError");
			}

			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				request.setAttribute("originalRequest", "/AuthorizeNetManageProfiles.do");
				return setErrorMessageAndForward(baseForm, mapping, "errors.userNotLoggedIn", "notLoggedIn");
			}

			// Start work
			Customer customer = baseForm.getUser().getCustomer();
			AuthorizeNetAPIProcessor apiProcessor = (AuthorizeNetAPIProcessor) baseForm.getBusinessObjectFactory()
					.createObject("authorizeNetAPIProcessorImplementer");
			String cimProfileIDStr = customer.getCimProfileID();
			long cimProfileID = 0;
			try {
				cimProfileID = Long.parseLong(cimProfileIDStr);
			} catch (Exception ex) {
				cimProfileID = 0;
			}
			if (cimProfileID < 0) {

				// Admin has set this as an invalid payment method for this user
				return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesCustomerNotAllowed",
						"nonFatalError");
			} else if (cimProfileID == 0) {

				// Create a new profile
				try {
					Map profileResult = apiProcessor.createCustomerProfile(customer, null);
					cimProfileIDStr = (String) profileResult.get("newProfileID");
					cimProfileID = Long.parseLong(cimProfileIDStr);
				} catch (Exception e) {
					log.error("Exception creating a new customer profile: ", e);
					return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesCommunicationError",
							"nonFatalError");
				}

				// Make sure it was created
				if (StringUtils.isBlank(cimProfileIDStr)) {
					return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesUnableToCreateProfile",
							"nonFatalError");
				}
			}

			// If using the ServerToServer API method, the only thing we do is show the page as everything is done with
			// ajax.
			if (!"AcceptHostedIframe".equals(baseForm.getSettings().getValue("authorizeNetIntegrationMethod"))) {
				log.info("Returning after creating a cimProfileID - using ajax/ServerToServer");
				return mapping.findForward("success");
			}

			// Get a token for the hosted page.
			String accountHostedPageToken = null;
			try {
				Map parameters = new HashMap();
				parameters.put("cimProfileID", cimProfileIDStr);
				parameters.put("returnURL", "AuthorizeNetManageProfiles.do");
				Map hostedProfilePageResult = apiProcessor.hostedProfilePage(parameters);
				accountHostedPageToken = (String) hostedProfilePageResult.get("accountHostedPageToken");
			} catch (Exception e) {
				log.error("Exception getting a hosted token: ", e);
				return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesCommunicationError",
						"nonFatalError");
			}

			// Make sure it was created
			if (StringUtils.isBlank(accountHostedPageToken)) {
				log.error("Error getting a hosted token");
				return setErrorMessageAndForward(baseForm, mapping, "errors.cimManageProfilesCommunicationError",
						"nonFatalError");
			}
			baseForm.getRequest().setAttribute("accountHostedPageToken", accountHostedPageToken);

			// Retrieve the profiles from Authorize.net
			apiProcessor = baseForm.getBusinessObjectFactory().createObject("authorizeNetAPIProcessorImplementer");
			Collection<CustomerPaymentProfileMaskedType> paymentProfiles = apiProcessor
					.retrievePaymentProfiles(cimProfileIDStr, false);
			if (paymentProfiles != null && paymentProfiles.size() > 0) {
				baseForm.getRequest().setAttribute("paymentProfiles", paymentProfiles);
			} else {
				log.info("No payment profiles to show.");
			}

			// Process a delete request. This action handles deletes for hosted options.
			String authorizeNetAllowDeleteProfiles = baseForm.getSettings()
					.getValue("authorizeNetAllowDeleteProfiles");
			String deleteID = request.getParameter("deleteID");
			if (StringUtils.isNotBlank(deleteID) && "1".equals(authorizeNetAllowDeleteProfiles)) {
				for (CustomerPaymentProfileMaskedType p : paymentProfiles) {
					if (p.getCustomerPaymentProfileId().equals(deleteID)) {
						Map deleteResult = null;
						try {
							Map parameters = PropertyUtils.describe(baseForm);
							parameters.put("customerProfileID", customer.getCimProfileID());
							parameters.put("paymentProfileID", deleteID);
							deleteResult = apiProcessor.deletePaymentProfile(parameters);
						} catch (Exception ex) {
							log.error("Exception deleting payment profile: ", ex);
							baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
									new ActionMessage("errors.cimManageProfilesUnableToDeleteProfile"));
							saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
							return mapping.findForward("failure");
						}
						if (deleteResult == null || deleteResult.get("resultCode") == null
								|| ("Error").equalsIgnoreCase((String) deleteResult.get("resultCode"))) {
							log.error("Error deleting payment profile" + deleteResult.get("resultCode") + ": "
									+ deleteResult.get("resultMessage"));
							baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
									new ActionMessage("errors.cimManageProfilesUnableToDeleteProfile"));
							saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
							return mapping.findForward("failure");
						} else {

							// Remove it from the request scope variable so it isn't re-shown
							paymentProfiles.remove(p);
						}
						break;
					}
				}
			}
			baseForm.getRequest().setAttribute("paymentProfiles", paymentProfiles);
			return mapping.findForward("success");
		}
	}

	private ActionForward setErrorMessageAndForward(BaseForm baseForm, ActionMapping mapping, String code,
			String forward) {
		baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(code));
		saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
		if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
			log.info("Validation error(s): " + baseForm.getErrors().toString());
		return mapping.findForward(forward);
	}
}
