package com.softslate.commerce.daos.payment;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.payment.Payment;
import com.softslate.commerce.daos.core.BaseAdminGatewayDAOHibernate;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO that runs miscellaneous queries related to payments.
 * 
 * @author David Tobey
 */
public class PaymentGatewayDAOHibernate extends BaseAdminGatewayDAOHibernate implements PaymentGatewayDAO {

	static Log log = LogFactory.getLog(PaymentGatewayDAOHibernate.class);

	public Map loadPaymentsAndCount(Map parameters) throws Exception {
		return loadItemsAndCount(parameters);
	}

	public Map editDeletePayments(Map parameters, BusinessObjectFactory bof) throws Exception {
		return editDeleteItems(parameters, bof);
	}

	public Collection getPaymentsFromValues(String value1, String value2) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "payment.selectPaymentsFromValues");
			q.setString("value1", value1);
			q.setString("value2", value2);
			return new LinkedHashSet(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection loadCardInfoStatistics() throws DataAccessException {
		Collection stats = new ArrayList();
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "payment.selectRecentCardInfoCount");
			// Last 24 hours
			Calendar start = Calendar.getInstance();
			start.add(Calendar.HOUR_OF_DAY, -24);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			stats.add(q.uniqueResult());
			// Last 7 days
			start = Calendar.getInstance();
			start.add(Calendar.DATE, -7);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			stats.add(q.uniqueResult());
			// Last 30 days
			start = Calendar.getInstance();
			start.add(Calendar.DATE, -30);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			stats.add(q.uniqueResult());
			// Last 90 days
			start = Calendar.getInstance();
			start.add(Calendar.DATE, -90);
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			stats.add(q.uniqueResult());
			// All time
			Query q2 = getNamedQuery(session, "payment.selectAllCardInfoCount");
			stats.add(q2.uniqueResult());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
		return stats;
	}

	public int purgeCardInfo(boolean commit, Map parameters) throws DataAccessException {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "payment.purgeCreditCardInfo");
			Calendar start = Calendar.getInstance();
			start.add(Calendar.DATE, (-1 * Integer.parseInt((String) parameters.get("olderThanDays"))));
			start.add(Calendar.HOUR_OF_DAY, (-1 * Integer.parseInt((String) parameters.get("olderThanHours"))));
			q.setString("startDate", formatDateTime(new java.util.Date(start.getTimeInMillis())));
			int updateCount = q.executeUpdate();
			if (commit) {
				getDaoFactory().commitTransaction();
			}
			return updateCount;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public Collection<Payment> loadPaymentsWithNoCIMProfileByDate(String cutoffDate) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session, "payment.selectPaymentsWithNoCIMProfileByDate");
			q.setString("cutoffDate", cutoffDate);
			return new LinkedHashSet<Payment>(q.list());
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}

	public List<Object[]> loadPaymentsWithNoProfileByDate(String createdStart,
			String createdEnd) throws Exception {
		Session session = getDaoFactory().startSession();
		try {
			Query q = getNamedQuery(session,
					"payment.selectPaymentsWithNoProfileByDate");
			q.setString("createdStart", createdStart);
			q.setString("createdEnd", createdEnd);
			List<Object[]> list = q.list();
			return list;
		} catch (HibernateException he) {
			if (log.isErrorEnabled())
				log.error("Hibernate error: " + he.toString());
			throw new DataAccessException(he);
		}
	}
}
