(function ($) {

	$(document).ready(function() {
		if (SSCGlobalJSVariables.useAjaxForFacetedSearch == 'undefined' || SSCGlobalJSVariables.useAjaxForFacetedSearch == '1') {
			var widget = new FacetedSearchWidget("facetedSearch", "facetedSearchBody", "pageNumber", "sortBy");
			widget.initialize();
		}
	});

	var FacetedSearchWidget = function(widgetContainerID, productListContainerClass, sortByLinksClass, pageNumberClass) {
		this.widgetContainerID = widgetContainerID;
		this.productListContainerClass = productListContainerClass;
		this.sortByLinksClass = sortByLinksClass;
		this.pageNumberClass = pageNumberClass;
	};
	
	FacetedSearchWidget.prototype.initialize = function() {
		$('#' + this.widgetContainerID).find('input.drilldown:checked').removeAttr('checked');
		$('#' + this.widgetContainerID).find('input.drilldown:checked').removeAttr('disabled');
		$(".refineButton").hide();
		var widget = this;
		$('#' + this.widgetContainerID).find('input.drilldown').click(function() {
			widget.addHashTagToBrowserLocation($(this));
			widget.uncheckParentAndChildDrilldowns($(this));
			widget.lockClickingOtherDrilldowns();
			widget.refreshProductList(null);
			widget.refreshFacetCounts($(this));
			widget.releaseClickingOtherDrilldowns();
		})
	}
	
	FacetedSearchWidget.prototype.addHashTagToBrowserLocation = function(checkbox) {
		var facetBeingClicked = checkbox.val();
		var checked = checkbox.attr('checked');
		if (checked === undefined) {
			checked = 'unchecked';
		}
		location.hash = checked + '|' + facetBeingClicked;
	}
	
	FacetedSearchWidget.prototype.uncheckParentAndChildDrilldowns = function(checkbox) {
		checkbox.closest('.facet').parents('.facet').children('label').children('input.drilldown:checked').attr("checked", false);
		checkbox.closest('.facet').parents('.facet').children('input.drilldown:checked').attr("checked", false);
		checkbox.closest('.facet').children('ul').find('input.drilldown:checked').attr("checked", false);
	}
	
	FacetedSearchWidget.prototype.lockClickingOtherDrilldowns = function() {
		$('#' + this.widgetContainerID).find('.facetNameAndCount').addClass('waiting');
		$('#' + this.widgetContainerID).find('input.drilldown:checked').attr("disabled", true);
	}
	
	FacetedSearchWidget.prototype.refreshProductList = function(pageOrSortByLink) {
		var checkedValues = convertArrayOfInputElementsToArrayOfValues($('#' + this.widgetContainerID).find('input.drilldown:checked'));
		var parameters = { "ajax": "true", "facets": checkedValues };
		parameters = this.addSearchParamsFromPage(parameters);
		var ajaxUrl = $(location).attr('href');
		if (pageOrSortByLink != null) {
			ajaxUrl = pageOrSortByLink.attr('href');
		}
		var widget = this;
		$.ajax({
		  	url: ajaxUrl,
		  	type: "POST",
		  	dataType: "html",
		  	data: parameters,
		  	traditional: true,
		  	async: false,
		  	success: function(data) {
		  		$('.' + widget.productListContainerClass).html(data);
		  	}
		});
		this.capturePageAndSortClicks();
	}
	
	FacetedSearchWidget.prototype.capturePageAndSortClicks =  function(link) {
		var widget = this;
		$('.' + this.pageNumberClass).click(function(event) {
			event.preventDefault();
			widget.refreshProductList($(this));
		})
		$('.' + this.sortByLinksClass).click(function(event) {
			event.preventDefault();
			widget.refreshProductList($(this));
		})
	}
	
	FacetedSearchWidget.prototype.refreshFacetCounts = function(checkbox) {
		var facetBeingClicked = checkbox.val();
		var allCheckedFacets = convertArrayOfInputElementsToArrayOfValues($('#' + this.widgetContainerID).find('input.drilldown:checked'));
		var facetGroups = $(".facetGroup");
		var widget = this;
		facetGroups.each( function(i) { 
				widget.refreshFacetGroup($(this), facetBeingClicked, allCheckedFacets);
		})
	}	
	
	FacetedSearchWidget.prototype.refreshFacetGroup = function(facetGroup, facetBeingClicked, allCheckedFacets) {
		
		// No need to retrieve counts for the facet group currently being checked
		if (!startsWith(facetBeingClicked, facetGroup.attr('id'))) {
			var facetGroupBeingRefreshed = facetGroup;
			
			// When retrieving counts for another parent facet, don't include its child facets in the search
			var facetsToRestictBy = this.checkedFacetsMinusThoseForGivenParentFacet(allCheckedFacets, facetGroupBeingRefreshed);
			facetsToRestictBy.push(facetGroupBeingRefreshed.attr('id'));
			var parameters = { "ajax": "true", 
				"facetGroupBeingRefreshed": facetGroupBeingRefreshed.attr('id'),
				"facets": facetsToRestictBy
			};
			parameters = this.addSearchParamsFromPage(parameters);
			var widget = this;
			var ajaxUrl = SSCGlobalJSVariables.customerURL + "/FacetCounts.do";
			$.ajax({
			  	url: ajaxUrl,
			  	type: "POST",
			  	dataType: "json",
			  	data: parameters,
			  	traditional: true,
			  	async: false,
			  	success: function(data) { 
			  		widget.refreshCounts(data, facetGroupBeingRefreshed);
			  	}
			});
		}
	}
	
	FacetedSearchWidget.prototype.checkedFacetsMinusThoseForGivenParentFacet = function(checkedFacets, facetGroupBeingRefreshed) {
		var filteredFacets = [];
		for (var i = 0; i < checkedFacets.length; i++) {
			if (!startsWith(checkedFacets[i], facetGroupBeingRefreshed.attr('id'))) {
				filteredFacets.push(checkedFacets[i]);
			}
		}
		return filteredFacets;
	}
	
	FacetedSearchWidget.prototype.refreshCounts = function(data, facetGroupBeingRefreshed) {
		var facetsUnderFacetGroup = facetGroupBeingRefreshed.find(".facet");
		var atLeastOneVisibleFacetUnderFacetGroup = false;
		var widget = this;
		facetsUnderFacetGroup.each(function(i) { 
			var countMoreThanZero = widget.refreshCount(data, facetGroupBeingRefreshed, $(this));
			if (countMoreThanZero && !atLeastOneVisibleFacetUnderFacetGroup) {
				atLeastOneVisibleFacetUnderFacetGroup = true;
			}
		});
		if (atLeastOneVisibleFacetUnderFacetGroup) {
			facetGroupBeingRefreshed.show();
		} else {
			facetGroupBeingRefreshed.hide();
		}
	}
	
	FacetedSearchWidget.prototype.refreshCount = function(data, facetGroupBeingRefreshed, facet) {
		var countMoreThanZero = true;
		var facetId = facet.attr('id');
		if (startsWith(facetId, facetGroupBeingRefreshed.attr('id'))) {
			countMoreThanZero = false;
			var facetCount = data[facetId];
			if (facetCount != null) {
				var facetCountElement = facet.find(".facetCount");
				facetCountElement.html(facetCount);
				facet.show();
				countMoreThanZero = true;
			} else {
				facet.hide();
			}
		}
		return countMoreThanZero;
	}
	
	FacetedSearchWidget.prototype.releaseClickingOtherDrilldowns = function() {
		$('#' + this.widgetContainerID).find('input.drilldown:checked').removeAttr('disabled');
		$('#' + this.widgetContainerID).find('.facetNameAndCount').removeClass('waiting');
	}
	
	FacetedSearchWidget.prototype.addSearchParamsFromPage = function(parameters) {
		if (typeof SSCGlobalJSVariables.searchListType !== 'undefined') {
			parameters["type"] = SSCGlobalJSVariables.searchListType;
		}
		if (typeof SSCGlobalJSVariables.searchListSearchString !== 'undefined') {
			parameters["searchString"] = SSCGlobalJSVariables.searchListSearchString;
		}
		if (typeof SSCGlobalJSVariables.searchListCategoryID !== 'undefined') {
			parameters["categoryID"] = SSCGlobalJSVariables.searchListCategoryID;
		}
		if (typeof SSCGlobalJSVariables.searchListManufacturerID !== 'undefined') {
			parameters["manufacturerID"] = SSCGlobalJSVariables.searchListManufacturerID;
		}
		return parameters;
	}
	
	var convertArrayOfInputElementsToArrayOfValues = function(inputElements) {
		var values = new Array(inputElements.length);
		for (var i = 0; i < inputElements.length; i++) {
			values[i] = inputElements[i].value;
		}
		return values;
	}
	
	var startsWith = function(string1, string2) {
		return String(string1).length >= String(string2).length && String(string1).substring(0, String(string2).length) === string2;
	}
})(jQuery);
